# -*- coding: utf-8 -*-
"""

    mslib.mscolab.server
    ~~~~~~~~~~~~~~~~~~~~

    Server for mscolab module

    This file is part of MSS.

    :copyright: Copyright 2019 Shivashis Padhi
    :copyright: Copyright 2019-2022 by the MSS team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import argparse
import logging
import platform
import os
import shutil
import sys
import secrets

from mslib import __version__
from mslib.mscolab.conf import mscolab_settings
from mslib.mscolab.seed import seed_data, add_user, add_all_users_default_operation,\
    add_all_users_to_all_operations, delete_user
from mslib.mscolab.utils import create_files
from mslib.utils import setup_logging
from mslib.utils.qt import Worker, Updater


def handle_start(args):
    from mslib.mscolab.server import APP, initialize_managers, start_server
    setup_logging(args)
    logging.info("MSS Version: %s", __version__)
    logging.info("Python Version: %s", sys.version)
    logging.info("Platform: %s (%s)", platform.platform(), platform.architecture())
    logging.info("Launching user interface...")

    app, sockio, cm, fm = initialize_managers(APP)
    start_server(app, sockio, cm, fm)


def confirm_action(confirmation_prompt):
    while True:
        confirmation = input(confirmation_prompt).lower()
        if confirmation == "n" or confirmation == "":
            return False
        elif confirmation == "y":
            return True
        else:
            print("Invalid input! Please select an option between y or n")


def handle_db_init():
    from mslib.mscolab.server import APP, db
    create_files()
    with APP.app_context():
        db.create_all()
    print("Database initialised successfully!")


def handle_db_reset(verbose=True):
    from mslib.mscolab.server import APP, db
    if os.path.exists(mscolab_settings.DATA_DIR):
        shutil.rmtree(mscolab_settings.DATA_DIR)
    create_files()
    with APP.app_context():
        db.drop_all()
        db.create_all()
    if verbose is True:
        print("Database has been reset successfully!")


def handle_db_seed():
    handle_db_reset(verbose=False)
    seed_data()
    print("Database seeded successfully!")


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--version", help="show version", action="store_true", default=False)
    parser.add_argument("--update", help="Updates MSS to the newest version", action="store_true", default=False)

    subparsers = parser.add_subparsers(help='Available actions', dest='action')

    server_parser = subparsers.add_parser("start", help="Start the mscolab server")
    server_parser.add_argument("--debug", help="show debugging log messages on console", action="store_true",
                               default=False)
    server_parser.add_argument("--logfile", help="If set to a name log output goes to that file", dest="logfile",
                               default=None)

    database_parser = subparsers.add_parser("db", help="Manage mscolab database")
    database_parser = database_parser.add_mutually_exclusive_group(required=True)
    database_parser.add_argument("--init", help="Initialise database", action="store_true")
    database_parser.add_argument("--reset", help="Reset database", action="store_true")
    database_parser.add_argument("--seed", help="Seed database", action="store_true")
    database_parser.add_argument("--users_by_file", type=argparse.FileType('r'),
                                 help="adds users into database, fileformat: suggested_username  name   <email>")
    database_parser.add_argument("--delete_users_by_file", type=argparse.FileType('r'),
                                 help="removes users from the database, fileformat: email")
    database_parser.add_argument("--default_operation", help="adds all users into a default TEMPLATE operation",
                                 action="store_true")
    database_parser.add_argument("--add_all_to_all_operation", help="adds all users into all other operations",
                                 action="store_true")

    args = parser.parse_args()

    if args.version:
        print("***********************************************************************")
        print("\n            Mission Support System (MSS)\n")
        print("***********************************************************************")
        print("Documentation: http://mss.rtfd.io")
        print("Version:", __version__)
        sys.exit()

    updater = Updater()
    if args.update:
        updater.on_update_available.connect(lambda old, new: updater.update_mss())
        updater.on_log_update.connect(lambda s: print(s.replace("\n", "")))
        updater.on_status_update.connect(lambda s: print(s.replace("\n", "")))
        updater.run()
        while Worker.workers:
            list(Worker.workers)[0].wait()
        sys.exit()

    updater.on_update_available.connect(lambda old, new: logging.info(f"MSS can be updated from {old} to {new}.\nRun"
                                                                      " the --update argument to update the server."))
    updater.run()

    if args.action == "start":
        handle_start(args)

    elif args.action == "db":
        if args.init:
            handle_db_init()
        elif args.reset:
            confirmation = confirm_action("Are you sure you want to reset the database? This would delete "
                                          "all your data! (y/[n]):")
            if confirmation is True:
                handle_db_reset()
        elif args.seed:
            confirmation = confirm_action("Are you sure you want to seed the database? Seeding will delete all your "
                                          "existing data and replace it with seed data (y/[n]):")
            if confirmation is True:
                handle_db_seed()
        elif args.users_by_file is not None:
            # fileformat: suggested_username  name   <email>
            confirmation = confirm_action("Are you sure you want to add users to the database? (y/[n]):")
            if confirmation is True:
                for line in args.users_by_file.readlines():
                    info = line.split()
                    username = info[0]
                    emailid = info[-1][1:-1]
                    password = secrets.token_hex(8)
                    add_user(emailid, username, password)
        elif args.default_operation:
            confirmation = confirm_action(
                "Are you sure you want to add users to the default TEMPLATE operation? (y/[n]):")
            if confirmation is True:
                # adds all users as collaborator on the operation TEMPLATE if not added, command can be repeated
                add_all_users_default_operation(access_level='admin')
        elif args.add_all_to_all_operation:
            confirmation = confirm_action(
                "Are you sure you want to add users to the ALL operations? (y/[n]):")
            if confirmation is True:
                # adds all users to all Operations
                add_all_users_to_all_operations()
        elif args.delete_users_by_file:
            confirmation = confirm_action(
                "Are you sure you want to delete a user? (y/[n]):")
            if confirmation is True:
                # deletes users from the db
                for email in args.delete_users_by_file.readlines():
                    delete_user(email.strip())


if __name__ == '__main__':
    main()



# (mss)
from PyQt5 import QtWidgets, QtCore

# (msui)
from mslib.support.qt_json_view.datatypes import DataType, TypeRole


class JsonDelegate(QtWidgets.QStyledItemDelegate):
    """Display the data based on the definitions on the DataTypes."""

    def sizeHint(self, option, index):
        return QtCore.QSize(option.rect.width(), 20)

    def paint(self, painter, option, index):
        """Use method from the data type or fall back to the default."""
        if index.column() == 0:
            return super(JsonDelegate, self).paint(painter, option, index)
        type_ = index.data(TypeRole)
        if isinstance(type_, DataType):
            try:
                super(JsonDelegate, self).paint(painter, option, index)
                return type_.paint(painter, option, index)
            except NotImplementedError:
                pass
        return super(JsonDelegate, self).paint(painter, option, index)

    def createEditor(self, parent, option, index):
        """Use method from the data type or fall back to the default."""
        if index.column() == 0:
            return super(JsonDelegate, self).createEditor(
                parent, option, index)
        try:
            return index.data(TypeRole).createEditor(parent, option, index)
        except NotImplementedError:
            return super(JsonDelegate, self).createEditor(
                parent, option, index)

    def setModelData(self, editor, model, index):
        """Use method from the data type or fall back to the default."""
        if index.column() == 0:
            return super(JsonDelegate, self).setModelData(editor, model, index)
        try:
            return index.data(TypeRole).setModelData(editor, model, index)
        except NotImplementedError:
            return super(JsonDelegate, self).setModelData(editor, model, index)



# -*- coding: utf-8 -*-
"""

    mslib.mswms._tests.test_wms
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This module provides pytest functions to tests mswms.wms

    This file is part of MSS.

    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
    :copyright: Copyright 2017 Joern Ungermann
    :copyright: Copyright 2016-2022 by the MSS team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import os
from shutil import move

import mock
from nco import Nco
import pytest

import mslib.mswms.wms
import mslib.mswms.gallery_builder
import mslib.mswms.mswms as mswms
from importlib import reload
from mslib._tests.utils import callback_ok_image, callback_ok_xml, callback_ok_html, callback_404_plain
from mslib._tests.constants import DATA_DIR


class Test_WMS(object):
    def test_get_query_string_missing_parameters(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING': 'request=GetCapabilities'}

        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_404_plain(result.status, result.headers)
        assert isinstance(result.data, bytes), result

    def test_get_query_string_wrong_values(self):
        # version implemented is 1.1.1 and 1.3.0
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING': 'request=GetCapabilities&service=WMS&version=1.4.0'}

        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_404_plain(result.status, result.headers)
        assert isinstance(result.data, bytes), result

    def test_get_capabilities(self):
        cases = (
            {
                'wsgi.url_scheme': 'http',
                'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
                'QUERY_STRING': 'request=GetCapabilities&service=WMS&version=1.1.1'
            },
            {
                'wsgi.url_scheme': 'http',
                'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
                'QUERY_STRING': 'request=GetCapabilities&service=WMS&version=1.3.0'
            },
            {
                'wsgi.url_scheme': 'http',
                'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
                'QUERY_STRING': 'request=capabilities&service=WMS&version=1.1.1'
            },
            {
                'wsgi.url_scheme': 'http',
                'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
                'QUERY_STRING': 'request=capabilities&service=WMS&version=1.3.0'
            },
            {
                'wsgi.url_scheme': 'http',
                'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1',
                'HTTP_HOST': 'localhost:8081',
                'QUERY_STRING': 'request=capabilities&service=WMS&version'
            },
            {
                'wsgi.url_scheme': 'http',
                'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1',
                'HTTP_HOST': 'localhost:8081',
                'QUERY_STRING': 'request=capabilities&service=WMS'
            },
        )

        for tst_case in cases:
            self.client = mswms.application.test_client()
            result = self.client.get('/?{}'.format(tst_case["QUERY_STRING"]))
            callback_ok_xml(result.status, result.headers)
            assert isinstance(result.data, bytes), result

    def test_get_capabilities_lowercase(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING': 'request=getcapabilities&service=wms&version=1.1.1'}
        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_ok_xml(result.status, result.headers)
        assert isinstance(result.data, bytes), result

    def test_produce_hsec_plot(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.PLDiv01&styles=&elevation=200&srs=EPSG%3A4326&format=image%2Fpng&'
                'request=GetMap&bgcolor=0xFFFFFF&height=376&dim_init_time=2012-10-17T12%3A00%3A00Z&width=479&'
                'version=1.1.1&bbox=-50.0%2C20.0%2C20.0%2C75.0&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=application%2Fvnd.ogc.se_xml&transparent=FALSE'}
        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_ok_image(result.status, result.headers)

        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.PLDiv01&styles=&elevation=200&crs=EPSG%3A4326&format=image%2Fpng&'
                'request=GetMap&bgcolor=0xFFFFFF&height=376&dim_init_time=2012-10-17T12%3A00%3A00Z&width=479&'
                'version=1.3.0&bbox=20.0%2C-50.0%2C75.0%2C20.0&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=XML&transparent=FALSE'}
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_ok_image(result.status, result.headers)

    def test_produce_hsec_service_exception(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.PLDiv01&styles=&elevation=200&srs=EPSG%3A4326&format=image%2Fpng&'
                'request=GetMap&bgcolor=0xFFFFFF&height=376&dim_init_time=2012-10-17T12%3A00%3A00Z&width=479&'
                'version=1.1.1&bbox=-50.0%2C20.0%2C20.0%2C75.0&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=application%2Fvnd.ogc.se_xml&transparent=FALSE'}
        query_string = environ["QUERY_STRING"]
        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(query_string))
        callback_ok_image(result.status, result.headers)
        assert result.data.count(b"ServiceExceptionReport") == 0, result

        for orig, fake in [
                ("dim_init_time=2012-10-17T12%3A00%3A00Z", "dim_init_time=a20121017T12%3A00%3A00Z"),
                ("time=2012-10-17T12%3A00%3A00Z", "time=a20121-0-17T12%3A00%3A00Z"),
                ("time=2012-10-17T12%3A00%3A00Z", "time=2012-01-17T12%3A00%3A00Z"),
                ("&dim_init_time=2012-10-17T12%3A00%3A00Z", ""),
                ("&time=2012-10-17T12%3A00%3A00Z", ""),
                ("srs=EPSG%3A4326", "srs=EPSH%3A4326"),
                ("srs=EPSG%3A4326", "srs=EPSG%3AABCD"),
                ("srs=EPSG%3A4326", "srs=EPSG%3A6666"),
                ("ecmwf_EUR_LL015.PLDiv01", "PLDiv01"),
                ("ecmwf_EUR_LL015.PLDiv01", "ecmwf_EUR_LL015.PLDav01"),
                ("ecmwf_EUR_LL015.PLDiv01", "ecmwf_AUR_LL015.PLDiv01"),
                ("format=image%2Fpng", "format=omage%2Fpng"),
                ("bbox=-50.0%2C20.0%2C20.0%2C75.0", "bbox=-abcd%2C20.0%2C20.0%2C75.0")]:
            environ["QUERY_STRING"] = query_string.replace(orig, fake)
            result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
            callback_ok_xml(result.status, result.headers)
            assert result.data.count(b"ServiceExceptionReport") > 0, result

    def test_produce_vsec_plot(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.VS_HV01&styles=&srs=VERT%3ALOGP&format=image%2Fpng&'
                'request=GetMap&bgcolor=0xFFFFFF&height=245&dim_init_time=2012-10-17T12%3A00%3A00Z&width=842&'
                'version=1.1.1&bbox=201%2C500.0%2C10%2C100.0&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=application%2Fvnd.ogc.se_xml&path=52.78%2C-8.93%2C48.08%2C11.28&transparent=FALSE'}

        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_ok_image(result.status, result.headers)

        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.VS_HV01&styles=&crs=VERT%3ALOGP&format=image%2Fpng&'
                'request=GetMap&bgcolor=0xFFFFFF&height=245&dim_init_time=2012-10-17T12%3A00%3A00Z&width=842&'
                'version=1.3.0&bbox=201%2C500.0%2C10%2C100.0&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=XML&path=52.78%2C-8.93%2C48.08%2C11.28&transparent=FALSE'}

        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_ok_image(result.status, result.headers)

    def test_produce_vsec_service_exception(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.VS_HV01&styles=&srs=VERT%3ALOGP&format=image%2Fpng&'
                'request=GetMap&bgcolor=0xFFFFFF&height=245&dim_init_time=2012-10-17T12%3A00%3A00Z&width=842&'
                'version=1.1.1&bbox=201%2C500.0%2C10%2C100.0&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=application%2Fvnd.ogc.se_xml&path=52.78%2C-8.93%2C48.08%2C11.28&transparent=FALSE'}
        query_string = environ["QUERY_STRING"]

        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(query_string))
        callback_ok_image(result.status, result.headers)
        assert result.data.count(b"ServiceExceptionReport") == 0, result

        for orig, fake in [
                ("time=2012-10-17T12%3A00%3A00Z", "time=2012-01-17T12%3A00%3A00Z"),
                ("&dim_init_time=2012-10-17T12%3A00%3A00Z", ""),
                ("&time=2012-10-17T12%3A00%3A00Z", ""),
                ("layers=ecmwf_EUR_LL015.VS_HV01", "layers=ecmwf_AUR_LL015.VS_HV01"),
                ("layers=ecmwf_EUR_LL015.VS_HV01", "layers=ecmwf_EUR_LL015.VS_HV99"),
                ("format=image%2Fpng", "format=omage%2Fpng"),
                ("path=52.78%2C-8.93%2C48.08%2C11.28", "path=aaaa%2C-8.93%2C48.08%2C11.28"),
                ("&path=52.78%2C-8.93%2C48.08%2C11.28", ""),
                ("bbox=201%2C500.0%2C10%2C100.0", "bbox=aaa%2C500.0%2C10%2C100.0")]:
            environ["QUERY_STRING"] = query_string.replace(orig, fake)

            result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
            callback_ok_xml(result.status, result.headers)
            assert result.data.count(b"ServiceExceptionReport") > 0, result

    def test_produce_lsec_plot(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.LS_HV01&styles=&srs=LINE%3A1&format=text%2Fxml&'
                'request=GetMap&dim_init_time=2012-10-17T12%3A00%3A00Z&'
                'version=1.1.1&bbox=201&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=application%2Fvnd.ogc.se_xml&path=52.78%2C-8.93%2C25000%2C48.08%2C11.28%2C25000'}

        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_ok_xml(result.status, result.headers)

        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.LS_HV01&styles=&crs=LINE%3A1&format=text%2Fxml&'
                'request=GetMap&dim_init_time=2012-10-17T12%3A00%3A00Z&'
                'version=1.3.0&bbox=201&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=application%2Fvnd.ogc.se_xml&path=52.78%2C-8.93%2C25000%2C48.08%2C11.28%2C25000'}

        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_ok_xml(result.status, result.headers)

    def test_produce_lsec_service_exception(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.LS_HV01&styles=&srs=LINE%3A1&format=text%2Fxml&'
                'request=GetMap&dim_init_time=2012-10-17T12%3A00%3A00Z&'
                'version=1.1.1&bbox=201&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=application%2Fvnd.ogc.se_xml&path=52.78%2C-8.93%2C25000%2C48.08%2C11.28%2C25000'}
        query_string = environ["QUERY_STRING"]

        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(query_string))
        callback_ok_xml(result.status, result.headers)
        assert result.data.count(b"ServiceExceptionReport") == 0, result

        for orig, fake in [
                ("time=2012-10-17T12%3A00%3A00Z", "time=2012-01-17T12%3A00%3A00Z"),
                ("&dim_init_time=2012-10-17T12%3A00%3A00Z", ""),
                ("&time=2012-10-17T12%3A00%3A00Z", ""),
                ("layers=ecmwf_EUR_LL015.LS_HV01", "layers=ecmwf_AUR_LL015.LS_HV01"),
                ("layers=ecmwf_EUR_LL015.LS_HV01", "layers=ecmwf_EUR_LL015.LS_HV99"),
                ("format=text%2Fxml", "format=oext%2Fxml"),
                ("path=52.78%2C-8.93%2C25000%2C48.08%2C11.28%2C25000",
                 "path=aaaa%2C-8.93%2C25000%2C48.08%2C11.28%2C25000"),
                ("&path=52.78%2C-8.93%2C25000%2C48.08%2C11.28%2C25000", ""),
                ("bbox=201", "bbox=aaa")]:
            environ["QUERY_STRING"] = query_string.replace(orig, fake)

            result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
            callback_ok_xml(result.status, result.headers)
            assert result.data.count(b"ServiceExceptionReport") > 0, result

    def test_application_request(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.PLDiv01&styles=&elevation=200&srs=EPSG%3A4326&format=image%2Fpng&'
                'request=GetMap&bgcolor=0xFFFFFF&height=376&dim_init_time=2012-10-17T12%3A00%3A00Z&width=479&'
                'version=1.1.1&bbox=-50.0%2C20.0%2C20.0%2C75.0&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=application%2Fvnd.ogc.se_xml&transparent=FALSE'}

        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        assert isinstance(result.data, bytes), result

    def test_application_request_lowercase(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.PLDiv01&styles=&elevation=200&srs=EPSG%3A4326&format=image%2Fpng&'
                'request=getmap&bgcolor=0xFFFFFF&height=376&dim_init_time=2012-10-17T12%3A00%3A00Z&width=479&'
                'version=1.1.1&bbox=-50.0%2C20.0%2C20.0%2C75.0&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=application%2Fvnd.ogc.se_xml&transparent=FALSE'}

        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        assert isinstance(result.data, bytes), result

    def test_application_norequest(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING': '',
        }

        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_ok_html(result.status, result.headers)
        assert isinstance(result.data, bytes), result
        assert result.data.count(b"") >= 1, result

    def test_application_unkown_request(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING': 'request=abraham',
        }
        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_404_plain(result.status, result.headers)
        assert isinstance(result.data, bytes), result
        assert result.data.count(b"") > 0, result

    def test_multiple_images(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.PLDiv01,ecmwf_EUR_LL015.PLTemp01&styles=&elevation=200&'
                'srs=EPSG%3A4326&format=image%2Fpng&'
                'request=GetMap&bgcolor=0xFFFFFF&height=376&dim_init_time=2012-10-17T12%3A00%3A00Z&width=479&'
                'version=1.1.1&bbox=-50.0%2C20.0%2C20.0%2C75.0&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=application%2Fvnd.ogc.se_xml&transparent=FALSE'}

        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_ok_image(result.status, result.headers)
        assert isinstance(result.data, bytes), result

    def test_multiple_xml(self):
        environ = {
            'wsgi.url_scheme': 'http',
            'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
            'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.LS_HV01,ecmwf_EUR_LL015.LS_HV01&styles=&srs=LINE%3A1&format=text%2Fxml&'
                'request=GetMap&dim_init_time=2012-10-17T12%3A00%3A00Z&'
                'version=1.1.1&bbox=201&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=application%2Fvnd.ogc.se_xml&path=52.78%2C-8.93%2C25000%2C48.08%2C11.28%2C25000'}

        self.client = mswms.application.test_client()
        result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
        callback_ok_xml(result.status, result.headers)

    def test_import_error(self):
        with mock.patch.dict("sys.modules", {"mswms_settings": None, "mswms_auth": None}):
            reload(mslib.mswms.wms)
            assert mslib.mswms.wms.mswms_settings.__file__ is None
            assert mslib.mswms.wms.mswms_auth.__file__ is None
        reload(mslib.mswms.wms)
        assert mslib.mswms.wms.mswms_settings.__file__ is not None
        assert mslib.mswms.wms.mswms_auth.__file__ is not None

    def test_files_changed(self):
        def do_test():
            environ = {
                'wsgi.url_scheme': 'http',
                'REQUEST_METHOD': 'GET', 'PATH_INFO': '/', 'SERVER_PROTOCOL': 'HTTP/1.1', 'HTTP_HOST': 'localhost:8081',
                'QUERY_STRING':
                'layers=ecmwf_EUR_LL015.PLDiv01&styles=&elevation=200&crs=EPSG%3A4326&format=image%2Fpng&'
                'request=GetMap&bgcolor=0xFFFFFF&height=376&dim_init_time=2012-10-17T12%3A00%3A00Z&width=479&'
                'version=1.3.0&bbox=20.0%2C-50.0%2C75.0%2C20.0&time=2012-10-17T12%3A00%3A00Z&'
                'exceptions=XML&transparent=FALSE'}
            pl_file = next(file for file in os.listdir(DATA_DIR) if ".pl" in file)

            self.client = mswms.application.test_client()
            result = self.client.get('/?{}'.format(environ["QUERY_STRING"]))

            # Assert modified file was reloaded and now looks different
            nco = Nco()
            nco.ncap2(input=os.path.join(DATA_DIR, pl_file), output=os.path.join(DATA_DIR, pl_file),
                      options=["-s \"geopotential_height*=2\""])
            result2 = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
            nco.ncap2(input=os.path.join(DATA_DIR, pl_file), output=os.path.join(DATA_DIR, pl_file),
                      options=["-s \"geopotential_height/=2\""])
            assert result.data != result2.data

            # Assert moved file was reloaded and now looks like the first image
            move(os.path.join(DATA_DIR, pl_file), os.path.join(DATA_DIR, pl_file + "2"))
            result3 = self.client.get('/?{}'.format(environ["QUERY_STRING"]))
            move(os.path.join(DATA_DIR, pl_file + "2"), os.path.join(DATA_DIR, pl_file))
            assert result.data == result3.data

        with pytest.raises(AssertionError):
            do_test()

        watch_access = mslib.mswms.dataaccess.WatchModificationDataAccess(
            mslib.mswms.wms.mswms_settings._datapath, "EUR_LL015")
        watch_access.setup()
        with mock.patch.object(
            mslib.mswms.wms.server.hsec_layer_registry["ecmwf_EUR_LL015"]["PLDiv01"].driver,
            "data_access", new=watch_access):
            do_test()

    def test_gallery(self, tmpdir):
        tempdir = tmpdir.mkdir("static")
        docsdir = tmpdir.mkdir("docs")
        mslib.mswms.wms.STATIC_LOCATION = tempdir
        mslib.mswms.gallery_builder.STATIC_LOCATION = tempdir
        mslib.mswms.wms.DOCS_LOCATION = docsdir
        mslib.mswms.gallery_builder.DOCS_LOCATION = docsdir
        linear_plots = [[mslib.mswms.wms.server.lsec_drivers, mslib.mswms.wms.server.lsec_layer_registry]]

        mslib.mswms.wms.server.generate_gallery(generate_code=True, plot_list=linear_plots)
        assert os.path.exists(os.path.join(tempdir, "plots"))
        assert os.path.exists(os.path.join(tempdir, "code"))
        assert os.path.exists(os.path.join(tempdir, "plots.html"))
        mslib.mswms.gallery_builder.plot_htmls = {}

        mslib.mswms.wms.server.generate_gallery(generate_code=False, plot_list=linear_plots)
        assert not os.path.exists(os.path.join(tempdir, "code"))

        file = os.path.join(tempdir, "plots", os.listdir(os.path.join(tempdir, "plots"))[0])
        file2 = os.path.join(tempdir, "plots", os.listdir(os.path.join(tempdir, "plots"))[1])
        modified_at = os.path.getmtime(file2)
        os.remove(file)
        assert not os.path.exists(file)
        mslib.mswms.wms.server.generate_gallery(generate_code=False, plot_list=linear_plots)
        assert not os.path.exists(os.path.join(tempdir, "code"))
        assert os.path.exists(file), file
        assert modified_at == os.path.getmtime(file2), \
            (modified_at, os.path.getmtime(file2))

        mslib.mswms.wms.server.generate_gallery(clear=True, create=True, plot_list=linear_plots)
        assert modified_at != os.path.getmtime(file2)
        mslib.mswms.gallery_builder.plot_htmls = {}

        mslib.mswms.wms.server.generate_gallery(clear=True, generate_code=True, sphinx=True,
                                                plot_list=linear_plots)
        assert os.path.exists(os.path.join(docsdir, "plots"))
        assert os.path.exists(os.path.join(docsdir, "code"))
        assert os.path.exists(os.path.join(docsdir, "plots.html"))
        mslib.mswms.gallery_builder.plot_htmls = {}



# -*- coding: utf-8 -*-
"""

    mslib.mscolab._tests.test_mscolab
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    tests for mscolab functionalities

    This file is part of MSS.

    :copyright: Copyright 2019 Shivashis Padhi
    :copyright: Copyright 2019-2022 by the MSS team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
import os
from flask_testing import TestCase

from mslib.mscolab.conf import mscolab_settings
from mslib.mscolab.models import db, Operation, User, Permission
from mslib.mscolab.mscolab import handle_db_init, handle_db_reset, handle_db_seed
from mslib.mscolab.server import APP
from mslib.mscolab.seed import add_operation


class Test_Mscolab(TestCase):
    def create_app(self):
        app = APP
        app.config['SQLALCHEMY_DATABASE_URI'] = mscolab_settings.SQLALCHEMY_DB_URI
        app.config['MSCOLAB_DATA_DIR'] = mscolab_settings.MSCOLAB_DATA_DIR
        app.config['UPLOAD_FOLDER'] = mscolab_settings.UPLOAD_FOLDER
        app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
        app.config["TESTING"] = True
        app.config['LIVESERVER_TIMEOUT'] = 10
        app.config['LIVESERVER_PORT'] = 0
        return app

    def setUp(self):
        db.init_app(self.app)
        handle_db_init()
        assert Operation.query.all() == []
        assert User.query.all() == []
        assert Permission.query.all() == []

    def test_handle_db_reset(self):
        assert os.path.isdir(mscolab_settings.UPLOAD_FOLDER)
        assert os.path.isdir(mscolab_settings.MSCOLAB_DATA_DIR)
        all_operations = Operation.query.all()
        assert all_operations == []
        operation_name = "Example"
        assert add_operation(operation_name, "Test Example")
        assert os.path.isdir(os.path.join(mscolab_settings.MSCOLAB_DATA_DIR, operation_name))
        operation = Operation.query.filter_by(path=operation_name).first()
        assert operation.description == "Test Example"
        all_operations = Operation.query.all()
        assert len(all_operations) == 1
        handle_db_reset()
        # check operation dir name removed
        assert os.path.isdir(os.path.join(mscolab_settings.MSCOLAB_DATA_DIR, operation_name)) is False
        assert os.path.isdir(mscolab_settings.MSCOLAB_DATA_DIR)
        assert os.path.isdir(mscolab_settings.UPLOAD_FOLDER)
        # query db for operation_name
        operation = Operation.query.filter_by(path=operation_name).first()
        assert operation is None
        all_operations = Operation.query.all()
        assert all_operations == []

    def test_handle_db_seed(self):
        handle_db_reset()
        all_operations = Operation.query.all()
        assert all_operations == []
        handle_db_seed()
        all_operations = Operation.query.all()
        assert len(all_operations) == 6
        assert all_operations[0].path == "one"
        all_users = User.query.all()
        assert len(all_users) == 10
        all_permissions = Permission.query.all()
        assert len(all_permissions) == 17



# -*- coding: utf-8 -*-
"""

    mslib.mscolab._tests.test_file_manager
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    tests for file_manager functionalities

    This file is part of MSS.

    :copyright: Copyright 2020 Reimar Bauer
    :copyright: Copyright 2020-2022 by the MSS team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from flask_testing import TestCase
import os
import pytest

from mslib.mscolab.conf import mscolab_settings
from mslib.mscolab.models import Operation, db
from mslib.mscolab.server import APP
from mslib.mscolab.file_manager import FileManager
from mslib.mscolab.seed import add_user, get_user
from mslib.mscolab.mscolab import handle_db_reset


@pytest.mark.skipif(os.name == "nt",
                    reason="multiprocessing needs currently start_method fork")
class Test_FileManager(TestCase):
    render_templates = False

    def create_app(self):
        app = APP
        app.config['SQLALCHEMY_DATABASE_URI'] = mscolab_settings.SQLALCHEMY_DB_URI
        app.config['MSCOLAB_DATA_DIR'] = mscolab_settings.MSCOLAB_DATA_DIR
        app.config['UPLOAD_FOLDER'] = mscolab_settings.UPLOAD_FOLDER
        app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
        app.config["TESTING"] = True
        app.config['LIVESERVER_TIMEOUT'] = 10
        app.config['LIVESERVER_PORT'] = 0
        return app

    def setUp(self):
        handle_db_reset()
        db.init_app(self.app)
        self.userdata = 'UV10@uv10', 'UV10', 'uv10'
        self.anotheruserdata = 'UV20@uv20', 'UV20', 'uv20'
        self.fm = FileManager(self.app.config["MSCOLAB_DATA_DIR"])

        assert add_user(self.userdata[0], self.userdata[1], self.userdata[2])
        self.user = get_user(self.userdata[0])
        assert self.user is not None
        assert add_user(self.anotheruserdata[0], self.anotheruserdata[1], self.anotheruserdata[2])
        self.anotheruser = get_user(self.anotheruserdata[0])
        assert add_user('UV30@uv30', 'UV30', 'uv30')
        self.vieweruser = get_user('UV30@uv30')
        assert add_user('UV40@uv40', 'UV40', 'uv40')
        self.collaboratoruser = get_user('UV40@uv40')
        assert add_user('UV50@uv50', 'UV50', 'uv50')
        self.op2user = get_user('UV50@uv50')
        assert add_user('UV60@uv60', 'UV60', 'uv60')
        self.op2vieweruser = get_user('UV60@uv60')
        assert add_user('UV70@uv70', 'UV70', 'uv70')
        self.user1 = get_user('UV70@uv70')
        assert add_user('UV80@uv80', 'UV80', 'uv80')
        self.adminuser = get_user('UV80@uv80')
        self._example_data()

    def tearDown(self):
        pass

    def test_create_operation(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="famous")
            assert operation.path == flight_path
            assert self.fm.create_operation(flight_path, "something to know", self.user) is False
            flight_path, operation = self._create_operation(flight_path="example_flight_path", content=self.content1)
            assert operation.path == flight_path

    def test_get_operation_details(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path='operation2')
            pd = self.fm.get_operation_details(operation.id, self.user)
            assert pd['description'] == operation.description
            assert pd['path'] == operation.path
            assert pd['id'] == operation.id

    def test_list_operations(self):
        with self.app.test_client():
            self.fm.create_operation("first", "info about first", self.user)
            self.fm.create_operation("second", "info about second", self.user)
            expected_result = [{'access_level': 'creator',
                                'category': 'default',
                                'description': 'info about first',
                                'op_id': 1,
                                'path': 'first'},
                               {'access_level': 'creator',
                                'category': 'default',
                                'description': 'info about second',
                                'op_id': 2,
                                'path': 'second'}]
            assert self.fm.list_operations(self.user) == expected_result

    def test_is_creator(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path='third')
            assert self.fm.is_creator(self.user.id, operation.id)

    def test_is_collaborator(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path='fourth')
            assert self.anotheruser.id is not None
            self.fm.add_bulk_permission(operation.id, self.user, [self.anotheruser.id], "collaborator")
            assert self.fm.is_collaborator(self.anotheruser.id, operation.id)

    def test_is_non_admin_member(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path='fifth')
            assert self.anotheruser.id is not None
            self.fm.add_bulk_permission(operation.id, self.user, [self.vieweruser.id], "viewer")
            assert self.vieweruser.id is not None
            assert self.fm.is_admin(self.vieweruser.id, operation.id) is False

    def test_is_viewer(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="test_flight")
            assert operation.path == flight_path
            self.fm.add_bulk_permission(operation.id, self.user, [self.vieweruser.id], "viewer")
            assert self.fm.is_admin(self.vieweruser.id, operation.id) is False
            assert self.fm.is_creator(self.vieweruser.id, operation.id) is False
            assert self.fm.is_collaborator(self.vieweruser.id, operation.id) is False
            assert self.fm.is_viewer(self.vieweruser.id, operation.id) is True
            assert self.fm.is_member(self.vieweruser.id, operation.id) is True

            # cross check with creator self.user
            assert self.fm.is_creator(self.user.id, operation.id) is True
            assert self.fm.is_collaborator(self.user.id, operation.id) is False
            assert self.fm.is_viewer(self.user.id, operation.id) is False

    def test_is_member(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="sunset")
            assert operation.path == flight_path
            assert self.fm.is_member(82322, operation.id) is False
            assert self.fm.is_member(self.user.id, operation.id) is True

    def test_auth_type(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="aa")
            assert self.fm.auth_type(self.user.id, operation.id) != "collaborator"
            assert self.fm.auth_type(self.user.id, operation.id) == "creator"

    def test_update_operation(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path='operation3')
            rename_to = "operation03"
            self.fm.update_operation(operation.id, "path", rename_to, self.user)
            ren_operation = Operation.query.filter_by(path=rename_to).first()
            assert ren_operation.id == operation.id
            assert ren_operation.path == rename_to

    def test_delete_file(self):
        # Todo rename "file" to operation
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path='operation4')
            assert self.fm.delete_file(operation.id, self.user)
            assert Operation.query.filter_by(path=flight_path).first() is None

    def test_get_authorized_users(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path='operation5')
            assert self.fm.get_authorized_users(operation.id) == [{'access_level': 'creator',
                                                                   'username': self.userdata[1]}]

    def test_save_file(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="operation6", content=self.content1)
            # nothing changed
            assert self.fm.save_file(operation.id, self.content1, self.user) is False
            assert self.fm.save_file(operation.id, self.content2, self.user)

    def test_get_file(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="operation7")
            assert self.fm.get_file(operation.id, self.user).startswith('<?xml version="1.0" encoding="utf-8"?>')

    def test_get_all_changes(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="operation8")
            assert self.fm.get_all_changes(operation.id, self.user) == []
            assert self.fm.save_file(operation.id, self.content1, self.user)
            assert self.fm.save_file(operation.id, self.content2, self.user)
            changes = self.fm.get_all_changes(operation.id, self.user)
            assert len(changes) == 2

    def test_get_change_content(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="operation8")
            assert self.fm.get_all_changes(operation.id, self.user) == []
            assert self.fm.save_file(operation.id, self.content1, self.user)
            assert self.fm.save_file(operation.id, self.content2, self.user)
            all_changes = self.fm.get_all_changes(operation.id, self.user)
            assert self.fm.get_change_content(all_changes[1]["id"]) == self.content1

    def test_set_version_name(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="operation8")
            assert self.fm.get_all_changes(operation.id, self.user) == []
            assert self.fm.save_file(operation.id, self.content1, self.user)
            assert self.fm.save_file(operation.id, self.content2, self.user)
            all_changes = self.fm.get_all_changes(operation.id, self.user)
            assert self.fm.set_version_name(all_changes[1]["id"], operation.id, self.user.id, "THIS")

    def test_undo(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="operation8")
            assert self.fm.get_all_changes(operation.id, self.user) == []
            assert self.fm.save_file(operation.id, self.content1, self.user)
            assert self.fm.save_file(operation.id, self.content2, self.user)
            all_changes = self.fm.get_all_changes(operation.id, self.user)
            assert self.fm.undo(all_changes[1]["id"], self.user)

    def test_fetch_users_without_permission(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="operation9")
            assert len(self.fm.fetch_users_without_permission(operation.id, self.user.id)) == 7

    def test_fetch_users_with_permission(self):
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="operation9")
            assert self.fm.fetch_users_with_permission(operation.id, self.user.id) == []

    def test_delete_bulk_permission_for_creators(self):
        with self.app.test_client():
            flight_path, operation1 = self._create_operation(flight_path="testflight", user=self.user)
            assert self.fm.is_creator(self.user.id, operation1.id)
            assert self.user.id is not None
            self.fm.add_bulk_permission(operation1.id, self.user, [self.anotheruser.id], "collaborator")
            assert self.fm.is_collaborator(self.anotheruser.id, operation1.id)
            self.fm.add_bulk_permission(operation1.id, self.user, [self.collaboratoruser.id], "collaborator")
            assert self.fm.is_collaborator(self.collaboratoruser.id, operation1.id)
            flight_path1, operation2 = self._create_operation(flight_path="testflight1", user=self.op2user)
            assert self.fm.is_creator(self.op2user.id, operation2.id)
            assert self.op2user.id is not None
            # user (creator of operation1) has no right leave operation1
            assert self.fm.delete_bulk_permission(operation1.id, self.user, [self.user.id]) is False
            # user (creator of operation1) has the right to remove anotheruser from the operation1
            assert self.fm.delete_bulk_permission(operation1.id, self.user, [self.anotheruser.id]) is True
            # op2user (creator of operation2) has no right to remove collaboratoruser of operation1 from operation1
            assert self.fm.delete_bulk_permission(operation1.id, self.op2user, [self.collaboratoruser.id]) is False
            # op2user (creator of operation2) has no right to remove anotheruser of operation1 from operation2
            assert self.fm.delete_bulk_permission(operation2.id, self.op2user, [self.anotheruser.id]) is False

    def test_delete_bulk_permission_for_members(self):
        with self.app.test_client():
            flight_path, operation1 = self._create_operation(flight_path="testflight", user=self.user)
            assert self.fm.is_creator(self.user.id, operation1.id)
            assert self.user.id is not None
            self.fm.add_bulk_permission(operation1.id, self.user, [self.anotheruser.id], "collaborator")
            assert self.fm.is_collaborator(self.anotheruser.id, operation1.id)
            self.fm.add_bulk_permission(operation1.id, self.user, [self.collaboratoruser.id], "collaborator")
            assert self.fm.is_collaborator(self.collaboratoruser.id, operation1.id)
            self.fm.add_bulk_permission(operation1.id, self.user, [self.vieweruser.id], "viewer")
            assert self.fm.is_viewer(self.vieweruser.id, operation1.id)
            flight_path1, operation2 = self._create_operation(flight_path="testflight1", user=self.op2user)
            assert self.fm.is_creator(self.op2user.id, operation2.id)
            assert self.op2user.id is not None
            self.fm.add_bulk_permission(operation2.id, self.op2user, [self.op2vieweruser.id], "collaborator")
            # collaboratoruser of operation1 has no right to remove vieweruser from the operation1
            assert self.fm.delete_bulk_permission(operation1.id, self.collaboratoruser, [self.vieweruser.id]) is False
            # The below assertion fails in stable 6.1
            # The change is so that vieweruser (any user other than creator) can leave the operation
            # vieweruser of operation1 has the right to leave operation1
            assert self.fm.delete_bulk_permission(operation1.id, self.vieweruser, [self.vieweruser.id]) is True
            # collaboratoruser of operation1 has no right to remove op2vieweruser of operation2 from operation2
            assert self.fm.delete_bulk_permission(
                operation2.id, self.collaboratoruser, [self.op2vieweruser.id]
            ) is False
            # op2vieweruser of operation2 has no right to remove collaboratoruser of operation1 from operation2
            assert self.fm.delete_bulk_permission(
                operation2.id, self.op2vieweruser, [self.collaboratoruser.id]
            ) is False

    def test_delete_bulk_permission_for_non_members(self):
        with self.app.test_client():
            flight_path, operation1 = self._create_operation(flight_path="testflight", user=self.user)
            assert self.fm.is_creator(self.user.id, operation1.id)
            assert self.user.id is not None
            self.fm.add_bulk_permission(operation1.id, self.user, [self.collaboratoruser.id], "collaborator")
            assert self.fm.is_collaborator(self.collaboratoruser.id, operation1.id)
            # user1 (not a member of operation1) has no right to remove collaboratoruser of operation1 from operation1
            assert self.fm.delete_bulk_permission(operation1.id, self.user1, [self.collaboratoruser.id]) is False
            # collaboratoruser of operation1 has no right to remove user1 (not a member of operation1) from operation1
            assert self.fm.delete_bulk_permission(operation1.id, self.collaboratoruser, [self.user1.id]) is False

    def test_delete_bulk_permission_for_admin_members(self):
        # creator is the one who creates the operation
        # admin is a role to add users.
        # the creator has also this role. But there could be more admins
        # admins have the right to remove themselves
        with self.app.test_client():
            flight_path, operation = self._create_operation(flight_path="saturn")
            assert operation.path == flight_path
            self.fm.add_bulk_permission(operation.id, self.user, [self.adminuser.id], "admin")
            assert self.fm.is_admin(self.adminuser.id, operation.id) is True
            assert self.fm.delete_bulk_permission(operation.id, self.user, [self.user.id]) is False
            assert self.fm.delete_bulk_permission(operation.id, self.adminuser, [self.adminuser.id]) is True

    def test_import_permission(self):
        with self.app.test_client():
            flight_path10, operation10 = self._create_operation(flight_path="operation10")
            flight_path11, operation11 = self._create_operation(flight_path="operation11")
            flight_path12, operation12 = self._create_operation(flight_path="operation12", user=self.anotheruser)
            flight_path13, operation13 = self._create_operation_with_users(flight_path="operation13")
            flight_path14, operation14 = self._create_operation_with_users(flight_path="operation14")
            flight_path15, operation15 = self._create_operation_with_opposite_permissions(flight_path="operation15")
            # equal permissions, nothing to do
            result = (False, None, 'Permissions are already given')
            assert self.fm.import_permissions(operation10.id, operation11.id, self.user.id) == result
            # no admin or creator rights
            result = (False, None, 'Not the creator or admin of this operation')
            assert self.fm.import_permissions(operation10.id, operation12.id, self.user.id) == result
            # not a member
            result = (False, None, 'Not a member of this operation')
            assert self.fm.import_permissions(operation12.id, operation10.id, self.user.id) == result
            # we add to op8 all users of op11
            result = (True, {'add_users': [self.vieweruser.id, self.collaboratoruser.id],
                             'delete_users': [],
                             'modify_users': []}, 'success')
            assert self.fm.import_permissions(operation13.id, operation10.id, self.user.id) == result
            # we remove all users from op8 which are not in op9
            result = (True, {'add_users': [],
                             'delete_users': [self.vieweruser.id, self.collaboratoruser.id],
                             'modify_users': []}, 'success')
            assert self.fm.import_permissions(operation11.id, operation10.id, self.user.id) == result
            # we modify access level
            result = (True, {'add_users': [],
                             'delete_users': [],
                             'modify_users': [self.vieweruser.id, self.collaboratoruser.id]}, 'success')
            assert self.fm.import_permissions(operation15.id, operation14.id, self.user.id) == result

    def _example_data(self):
        self.content1 = """\
<?xml version="1.0" encoding="utf-8"?>
  <FlightTrack>
    <Name>new flight track (1)</Name>
    <ListOfWaypoints>
      <Waypoint flightlevel="0.0" lat="55.15" location="B" lon="-23.74">
        <Comments>Takeoff</Comments>
      </Waypoint>
      <Waypoint flightlevel="350" lat="42.99" location="A" lon="-12.1">
        <Comments></Comments>
      </Waypoint>
      <Waypoint flightlevel="380.0" lat="52.785" location="Shannon" lon="-8.925">
        <Comments>Dive</Comments>
      </Waypoint>
      <Waypoint flightlevel="400.0" lat="48.08" location="EDMO" lon="11.28">
        <Comments></Comments>
      </Waypoint>
      <Waypoint flightlevel="0.0" lat="63.74" location="C" lon="1.73">
        <Comments>Landing</Comments>
      </Waypoint>
    </ListOfWaypoints>
  </FlightTrack>"""
        self.content2 = """\
<?xml version="1.0" encoding="utf-8"?>
  <FlightTrack>
    <Name>new flight track (1)</Name>
    <ListOfWaypoints>
      <Waypoint flightlevel="0.0" lat="55.15" location="B" lon="-23.74">
        <Comments>Takeoff</Comments>
      </Waypoint>
      <Waypoint flightlevel="350" lat="42.99" location="A" lon="-12.1">
        <Comments></Comments>
      </Waypoint>
      </ListOfWaypoints>
  </FlightTrack>"""

    def _create_operation(self, flight_path="firstflight", user=None, content=None):
        if user is None:
            user = self.user
        self.fm.create_operation(flight_path, f"info about {flight_path}", user, content=content)
        operation = Operation.query.filter_by(path=flight_path).first()
        return flight_path, operation

    def _create_operation_with_users(self, flight_path="firstflight", user=None, content=None):
        if user is None:
            user = self.user
        self.fm.create_operation(flight_path, f"info about {flight_path}", user, content=content)
        operation = Operation.query.filter_by(path=flight_path).first()
        self.fm.add_bulk_permission(operation.id, self.user, [self.vieweruser.id], "viewer")
        self.fm.add_bulk_permission(operation.id, self.user, [self.collaboratoruser.id], "collaborator")
        return flight_path, operation

    def _create_operation_with_opposite_permissions(self, flight_path="firstflight", user=None, content=None):
        if user is None:
            user = self.user
        self.fm.create_operation(flight_path, f"info about {flight_path}", user, content=content)
        operation = Operation.query.filter_by(path=flight_path).first()
        self.fm.add_bulk_permission(operation.id, self.user, [self.vieweruser.id], "collaborator")
        self.fm.add_bulk_permission(operation.id, self.user, [self.collaboratoruser.id], "viewer")
        return flight_path, operation



# -*- coding: utf-8 -*-
"""

    mslib.mscolab._tests.test_utils
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    tests for mscolab/utils

    This file is part of MSS.

    :copyright: Copyright 2019 Shivashis Padhi
    :copyright: Copyright 2019-2022 by the MSS team, see AUTHORS.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from flask_testing import TestCase
import os
import pytest
import json

from fs.tempfs import TempFS
from mslib.mscolab.conf import mscolab_settings
from mslib.mscolab.models import db, Operation, MessageType
from mslib.mscolab.mscolab import handle_db_init, handle_db_reset
from mslib.mscolab.server import APP
from mslib.mscolab.seed import add_user, get_user
from mslib.mscolab.utils import get_recent_op_id, get_session_id, get_message_dict, create_files, os_fs_create_dir
from mslib.mscolab.sockets_manager import setup_managers


class Message():
    id = 1
    u_id = 2

    class user():
        username = "name"
    text = "Moin"
    message_type = MessageType.TEXT
    reply_id = 0
    replies = []

    class created_at():
        def strftime(value):
            pass


@pytest.mark.skipif(os.name == "nt",
                    reason="multiprocessing needs currently start_method fork")
class Test_Utils(TestCase):
    render_templates = False

    def create_app(self):
        app = APP
        app.config['SQLALCHEMY_DATABASE_URI'] = mscolab_settings.SQLALCHEMY_DB_URI
        app.config['MSCOLAB_DATA_DIR'] = mscolab_settings.MSCOLAB_DATA_DIR
        app.config['UPLOAD_FOLDER'] = mscolab_settings.UPLOAD_FOLDER
        app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
        app.config["TESTING"] = True
        app.config['LIVESERVER_TIMEOUT'] = 10
        app.config['LIVESERVER_PORT'] = 0
        return app

    def setUp(self):
        handle_db_init()
        self.userdata = 'UV10@uv10', 'UV10', 'uv10'
        self.anotheruserdata = 'UV20@uv20', 'UV20', 'uv20'
        socketio, cm, self.fm = setup_managers(self.app)

    def tearDown(self):
        handle_db_reset()

    @pytest.mark.skipif(os.name == "nt",
                        reason="multiprocessing needs currently start_method fork")
    def test_get_recent_oid(self):
        assert add_user(self.userdata[0], self.userdata[1], self.userdata[2])
        assert add_user(self.anotheruserdata[0], self.anotheruserdata[1], self.anotheruserdata[2])
        with self.app.test_client() as test_client:
            db.init_app(self.app)
            user = get_user(self.userdata[0])
            anotheruser = get_user(self.anotheruserdata[0])
            operation, token = self._create_operation(test_client, self.userdata)
            op_id = get_recent_op_id(self.fm, user)
            assert op_id == operation.id
            op_id = get_recent_op_id(self.fm, anotheruser)
            assert op_id is None

    def test_get_session_id(self):
        sockets = [{"u_id": 5, "s_id": 100}]
        assert get_session_id(sockets, 5) == 100

    def test_get_message_dict(self):
        result = get_message_dict(Message())
        assert result["message_type"] == MessageType.TEXT

    def test_os_fs_create_dir(self):
        _fs = TempFS(identifier="msui")
        _dir = _fs.getsyspath("")
        os_fs_create_dir(_dir)
        assert os.path.exists(_dir)

    def test_create_file(self):
        create_files()
        # ToDo refactor to fs
        assert os.path.exists(mscolab_settings.MSCOLAB_DATA_DIR)
        assert os.path.exists(mscolab_settings.UPLOAD_FOLDER)

    def _create_operation(self, test_client, userdata=None, path="firstflight", description="simple test"):
        if userdata is None:
            userdata = self.userdata
        response = test_client.post('/token', data={"email": userdata[0], "password": userdata[2]})
        data = json.loads(response.data.decode('utf-8'))
        token = data["token"]
        response = test_client.post('/create_operation', data={"token": token,
                                                               "path": path,
                                                               "description": description})
        assert response.status_code == 200
        assert response.data.decode('utf-8') == "True"
        operation = Operation.query.filter_by(path=path).first()
        return operation, token



# -*- coding: utf-8 -*-
"""

    mslib.utils._tests.test_qt
    ~~~~~~~~~~~~~~~~~~~~~~~~~~

    This module provides pytest functions for mslib.utils.qt

    This file is part of MSS.

    :copyright: Copyright 2017 Joern Ungermann
    :copyright: Copyright 2017-2022 by the MSS team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import mslib.utils.qt as mqt
import PyQt5 as pqt


def test_variant():
    for test_val in [-12.2, 2, 0, 12.2]:
        var = pqt.QtCore.QVariant(test_val)
        val = var.value()
        assert isinstance(val, (int, float))
        assert abs(val - test_val) < 1e-6

    for test_val in ["-12.2", "2", "0", u"12.2", "abc", u"aöc"]:
        var = pqt.QtCore.QVariant(test_val)
        val = var.value()
        assert val == test_val


def test_localized_conversion():
    value, ok = pqt.QtCore.QLocale(pqt.QtCore.QLocale.English).toDouble("12.2")
    assert ok is True
    assert value == 12.2
    value, ok = pqt.QtCore.QLocale(pqt.QtCore.QLocale.German).toDouble("12,2")
    assert ok is True
    assert value == 12.2
    value, ok = pqt.QtCore.QLocale(pqt.QtCore.QLocale.German).toDouble("1.200")
    assert ok is True
    assert value == 1200
    value, ok = pqt.QtCore.QLocale(pqt.QtCore.QLocale.French).toDouble("12,2")
    assert ok is True
    assert value == 12.2


def test_variant_to_string():
    for value, variant in [("5", "5"), ("5", "5"), (u"öäü", u"öäü"), ("abc", "abc")]:
        conv_value = mqt.variant_to_string(pqt.QtCore.QVariant(variant))
        assert value == conv_value


def test_variant_to_float():
    for value, variant in [(5, "5"), (5, 5), (5.5, 5.5), (-5.5, -5.5)]:
        conv_value = mqt.variant_to_float(pqt.QtCore.QVariant(variant))
        assert value == conv_value

    german_locale = pqt.QtCore.QLocale(pqt.QtCore.QLocale.German)
    for value, string in [(5, "5"), (5.5, "5,5"), (1000, "1.000")]:
        conv_value = mqt.variant_to_float(pqt.QtCore.QVariant(string), locale=german_locale)
        assert conv_value == value
    french_locale = pqt.QtCore.QLocale(pqt.QtCore.QLocale.French)
    for value, string in [(5, "5"), (5.5, "5,5"), (1000, "1 000")]:
        conv_value = mqt.variant_to_float(pqt.QtCore.QVariant(string), locale=french_locale)
        assert conv_value == value
    english_locale = pqt.QtCore.QLocale(pqt.QtCore.QLocale.English)
    for value, string in [(5, "5"), (5.5, "5.5"), (1000, "1,000")]:
        conv_value = mqt.variant_to_float(pqt.QtCore.QVariant(string), locale=english_locale)
        assert conv_value == value



# -*- coding: utf-8 -*-
"""

    mslib.utils.units
    ~~~~~~~~~~~~~~

    Collection of unit conversion related routines for the Mission Support System.

    This file is part of MSS.

    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr)
    :copyright: Copyright 2016-2022 by the MSS team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import logging

from metpy.units import units, check_units  # noqa
import pint

units.define("PVU = m^2 s^-1 uK kg^-1")
units.define("degrees_south = -degrees = degrees_S = degreesS = degree_south = degree_S = degreeS")
units.define("degrees_west = -degrees = degrees_W = degreesW = degree_west = degree_W = degreeW")

# Follow metpy 1.0.1 style of defining dimensionless units
for _name, _factor in (("level", 1),
                       ("sigma", 1),
                       ("permille", 1e-3),
                       ("ppm", 1e-6),
                       ("ppmv", 1e-6),
                       ("ppb", 1e-9),
                       ("ppbv", 1e-9),
                       ("ppt", 1e-12),
                       ("pptv", 1e-12)):
    units.define(pint.unit.UnitDefinition(_name, '', (),
                 pint.converters.ScaleConverter(_factor)))


def convert_to(value, from_unit, to_unit, default=1.):
    try:
        value_unit = units.Quantity(value, from_unit)
        result = value_unit.to(to_unit).magnitude
    except pint.UndefinedUnitError:
        logging.error("Error in unit conversion (undefined) '%s'/'%s'", from_unit, to_unit)
        result = value * default
    except pint.DimensionalityError:
        if units(to_unit).to_base_units().units == units.m:
            try:
                result = (value_unit / units.Quantity(9.81, "m s^-2")).to(to_unit).magnitude
            except pint.DimensionalityError:
                logging.error("Error in unit conversion (dimensionality) %s/%s", from_unit, to_unit)
                result = value * default
        else:
            logging.error("Error in unit conversion (dimensionality) %s/%s", from_unit, to_unit)
            result = value * default
    return result



# -*- coding: utf-8 -*-
"""

    mslib.msui.constants
    ~~~~~~~~~~~~~~~~~~~~

    This module provides constants

    This file is part of MSS.

    :copyright: Copyright 2008-2014 Deutsches Zentrum fuer Luft- und Raumfahrt e.V.
    :copyright: Copyright 2011-2014 Marc Rautenhaus (mr), Tongxi Lou (tl)
    :copyright: Copyright 2016-2017 Reimar Bauer
    :copyright: Copyright 2016-2022 by the MSS team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import fs
import os
import logging

# ToDo refactor to generic functions, keep only constants
HOME = os.path.expanduser(f"~{os.path.sep}")
MSUI_CONFIG_PATH = os.getenv("MSUI_CONFIG_PATH", os.path.join(HOME, ".config", "msui"))
if '://' in MSUI_CONFIG_PATH:
    try:
        _fs = fs.open_fs(MSUI_CONFIG_PATH)
    except fs.errors.CreateFailed:
        _fs.makedirs(MSUI_CONFIG_PATH)
    except fs.opener.errors.UnsupportedProtocol:
        logging.error(f'FS url "{MSUI_CONFIG_PATH}" not supported')
else:
    _dir = os.path.expanduser(MSUI_CONFIG_PATH)
    if not os.path.exists(_dir):
        os.makedirs(_dir)

GRAVATAR_DIR_PATH = fs.path.join(MSUI_CONFIG_PATH, "gravatars")

MSUI_SETTINGS = os.getenv('MSUI_SETTINGS', os.path.join(MSUI_CONFIG_PATH, "msui_settings.json"))

# We try to create an empty MSUI_SETTINGS file if not existing
# but there can be a permission problem
if '://' in MSUI_SETTINGS:
    dir_path, file_name = fs.path.split(MSUI_SETTINGS)
    try:
        _fs = fs.open_fs(dir_path)
        if not _fs.exists(file_name):
            with _fs.open(file_name, 'w') as fid:
                fid.write("{}")
    except fs.errors.CreateFailed:
        logging.error(f'"{MSUI_SETTINGS}" can''t be created')
else:
    if not os.path.exists(MSUI_SETTINGS):
        try:
            with open(MSUI_SETTINGS, 'w') as fid:
                fid.write("{}")
        except IOError:
            logging.error(f'"{MSUI_SETTINGS}" can''t be created')

WMS_LOGIN_CACHE = {}
MSC_LOGIN_CACHE = {}

POSIX = {"application_destination": os.path.join(HOME, ".local/share/applications/msui{}.desktop"),
         "icon_destination": os.path.join(HOME, ".local/share/icons/hicolor/{}/apps/mss-logo{}.png"),
         "desktop": """[Desktop Entry]
Name=msui {}
Comment=A web service based tool to plan atmospheric research flights (mission support system).
Keywords=documentation;information;
Exec={}
Icon={}
Type=Application
Categories=Science;Education;
StartupNotify=true
X-GNOME-SingleWindow=false
X-Ubuntu-Gettext-Domain=msui
"""}



# -*- coding: utf-8 -*-
"""

    mslib.msui.mscolab_change_history
    ~~~~~~~~~~~~~~~~~~~~~

    Mscolab change history window to display the change history of the flight path so that users
    can revert back to a previous version

    This file is part of MSS.

    :copyright: 2020 Tanish Grover
    :copyright: Copyright 2020-2022 by the MSS team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from datetime import datetime
import json

import requests
from werkzeug.urls import url_encode, url_join

from mslib.utils.verify_user_token import verify_user_token
from mslib.msui.flighttrack import WaypointsTableModel
from PyQt5 import QtCore, QtWidgets, QtGui
from mslib.msui.qt5 import ui_mscolab_version_history as ui
from mslib.utils.qt import show_popup
from mslib.utils.config import config_loader
from mslib.utils.time import utc_to_local_datetime


class MSColabVersionHistory(QtWidgets.QMainWindow, ui.Ui_MscolabVersionHistory):
    """Derives QMainWindow to provide some common functionality to all
       MSUI view windows.
    """
    name = "MSColab Version History Window"
    identifier = None
    viewCloses = QtCore.pyqtSignal(name="viewCloses")
    reloadWindows = QtCore.pyqtSignal(name="reloadWindows")

    def __init__(self, token, op_id, user, operation_name, conn, parent=None,
                 mscolab_server_url=config_loader(dataset="default_MSCOLAB")):
        """
        token: access_token
        op_id: operation id
        user: logged in user
        operation_name: name of operation,
        conn: socket connection
        parent: parent of widget
        mscolab_server_url: server url of mscolab
        """
        super(MSColabVersionHistory, self).__init__(parent)
        self.setupUi(self)
        # Initialise Variables
        self.token = token
        self.op_id = op_id
        self.user = user
        self.operation_name = operation_name
        self.conn = conn
        self.mscolab_server_url = mscolab_server_url

        # Event handlers
        self.refreshBtn.clicked.connect(self.handle_refresh)
        self.checkoutBtn.clicked.connect(self.handle_undo)
        self.nameVersionBtn.clicked.connect(self.handle_named_version)
        self.deleteVersionNameBtn.clicked.connect(self.handle_delete_version_name)
        self.versionFilterCB.currentIndexChanged.connect(lambda: self.load_all_changes())
        self.changes.currentItemChanged.connect(self.preview_change)
        # Setup UI
        self.deleteVersionNameBtn.setVisible(False)
        self.set_label_text()
        self.set_change_list_style()
        self.toggle_version_buttons(False)
        self.load_current_waypoints()
        self.load_all_changes()

    def set_label_text(self):
        self.usernameLabel.setText(f"Logged in: {self.user['username']}")
        self.operationNameLabel.setText(f"Operation: {self.operation_name}")

    def set_change_list_style(self):
        palette = self.changes.palette()
        self.changes.setStyleSheet(f"""
            QListWidget::item {{
                border-bottom: 1px solid #222;
            }}
            QListWidget::item:selected {{
                background-color: {palette.highlight().color().name()};
                color: {palette.highlightedText().color().name()};
            }}
        """)

    def toggle_version_buttons(self, state):
        self.checkoutBtn.setEnabled(state)
        self.nameVersionBtn.setEnabled(state)

    def load_current_waypoints(self):
        if verify_user_token(self.mscolab_server_url, self.token):
            data = {
                "token": self.token,
                "op_id": self.op_id
            }
            url = url_join(self.mscolab_server_url, 'get_operation_by_id')
            res = requests.get(url, data=data)
            if res.text != "False":
                xml_content = json.loads(res.text)["content"]
                waypoint_model = WaypointsTableModel(name="Current Waypoints", xml_content=xml_content)
                self.currentWaypointsTable.setModel(waypoint_model)
            else:
                # this triggers disconnect
                self.conn.signal_reload.emit(self.op_id)
        else:
            # this triggers disconnect
            self.conn.signal_reload.emit(self.op_id)

    def load_all_changes(self):
        """
        get changes from api, clear listwidget, render them to ui
        """
        if verify_user_token(self.mscolab_server_url, self.token):
            data = {
                "token": self.token,
                "op_id": self.op_id
            }
            named_version_only = None
            if self.versionFilterCB.currentIndex() == 0:
                named_version_only = True
            query_string = url_encode({"named_version": named_version_only})
            url_path = f'get_all_changes?{query_string}'
            url = url_join(self.mscolab_server_url, url_path)
            r = requests.get(url, data=data)
            if r.text != "False":
                changes = json.loads(r.text)["changes"]
                self.changes.clear()
                for change in changes:
                    created_at = datetime.strptime(change["created_at"], "%Y-%m-%d, %H:%M:%S")
                    local_time = utc_to_local_datetime(created_at)
                    date = local_time.strftime('%d/%m/%Y')
                    time = local_time.strftime('%I:%M %p')
                    item_text = f'{change["username"]} made change on {date} at {time}'
                    if change["version_name"] is not None:
                        item_text = f'{change["version_name"]}\n{item_text}'
                    item = QtWidgets.QListWidgetItem(item_text, parent=self.changes)
                    item.id = change["id"]
                    item.version_name = change["version_name"]
                    self.changes.addItem(item)
            else:
                # this triggers disconnect
                self.conn.signal_reload.emit(self.op_id)
        else:
            # this triggers disconnect
            self.conn.signal_reload.emit(self.op_id)

    def preview_change(self, current_item, previous_item):
        if verify_user_token(self.mscolab_server_url, self.token):
            font = QtGui.QFont()
            if previous_item is not None:
                previous_item.setFont(font)

            if current_item is None:
                self.changePreviewTable.setModel(None)
                self.deleteVersionNameBtn.setVisible(False)
                self.toggle_version_buttons(False)
                return

            font.setBold(True)
            current_item.setFont(font)
            data = {
                "token": self.token,
                "ch_id": current_item.id
            }
            url = url_join(self.mscolab_server_url, 'get_change_content')
            res = requests.get(url, data=data)
            if res.text != "False":
                res = res.json()
                waypoint_model = WaypointsTableModel(xml_content=res["content"])
                self.changePreviewTable.setModel(waypoint_model)
                if current_item.version_name is not None:
                    self.deleteVersionNameBtn.setVisible(True)
                else:
                    self.deleteVersionNameBtn.setVisible(False)
                self.toggle_version_buttons(True)
            else:
                # this triggers disconnect
                self.conn.signal_reload.emit(self.op_id)
        else:
            # this triggers disconnect
            self.conn.signal_reload.emit(self.op_id)

    def request_set_version_name(self, version_name, ch_id):
        if verify_user_token(self.mscolab_server_url, self.token):
            data = {
                "token": self.token,
                "version_name": version_name,
                "ch_id": ch_id,
                "op_id": self.op_id
            }
            url = url_join(self.mscolab_server_url, 'set_version_name')
            res = requests.post(url, data=data)
            return res
        else:
            # this triggers disconnect
            self.conn.signal_reload.emit(self.op_id)

    def handle_named_version(self):
        if verify_user_token(self.mscolab_server_url, self.token):
            version_name, completed = QtWidgets.QInputDialog.getText(self, 'Version Name Dialog', 'Enter version name:')
            if completed is True:
                if len(version_name) > 255 or len(version_name) == 0:
                    show_popup(self, "Error", "Version name length has to be between 1 and 255")
                    return
                selected_item = self.changes.currentItem()
                res = self.request_set_version_name(version_name, selected_item.id)
                if res.text != "False":
                    res = res.json()
                    if res["success"] is True:
                        item_text = selected_item.text().split('\n')[-1]
                        new_text = f"{version_name}\n{item_text}"
                        selected_item.setText(new_text)
                        selected_item.version_name = version_name
                        self.deleteVersionNameBtn.setVisible(True)
                    else:
                        show_popup(self, "Error", res["message"])
                else:
                    # this triggers disconnect
                    self.conn.signal_reload.emit(self.op_id)
        else:
            # this triggers disconnect
            self.conn.signal_reload.emit(self.op_id)

    def handle_delete_version_name(self):
        if verify_user_token(self.mscolab_server_url, self.token):
            selected_item = self.changes.currentItem()
            res = self.request_set_version_name(None, selected_item.id)
            if res.text != "False":
                res = res.json()
                if res["success"] is True:
                    # Remove item if the filter is set to Named version
                    if self.versionFilterCB.currentIndex() == 0:
                        self.changes.takeItem(self.changes.currentRow())
                    # Remove name from item
                    else:
                        item_text = selected_item.text().split('\n')[-1]
                        selected_item.setText(item_text)
                        selected_item.version_name = None
                    self.deleteVersionNameBtn.setVisible(False)
                else:
                    show_popup(self, "Error", res["message"])
            else:
                # this triggers disconnect
                self.conn.signal_reload.emit(self.op_id)
        else:
            # this triggers disconnect
            self.conn.signal_reload.emit(self.op_id)

    def handle_undo(self):
        if verify_user_token(self.mscolab_server_url, self.token):
            qm = QtWidgets.QMessageBox
            ret = qm.question(self, self.tr("Undo"), "Do you want to checkout to this change?", qm.Yes, qm.No)
            if ret == qm.Yes:
                data = {
                    "token": self.token,
                    "ch_id": self.changes.currentItem().id
                }
                url = url_join(self.mscolab_server_url, 'undo')
                r = requests.post(url, data=data)
                if r.text != "False":
                    # reload windows
                    self.reloadWindows.emit()
                    self.load_current_waypoints()
                    self.load_all_changes()
                else:
                    # this triggers disconnect
                    self.conn.signal_reload.emit(self.op_id)
        else:
            # this triggers disconnect
            self.conn.signal_reload.emit(self.op_id)

    def handle_refresh(self):
        if verify_user_token(self.mscolab_server_url, self.token):
            self.load_current_waypoints()
            self.load_all_changes()
        else:
            # this triggers disconnect
            self.conn.signal_reload.emit(self.op_id)

    def closeEvent(self, event):
        self.viewCloses.emit()
        event.accept()



# -*- coding: utf-8 -*-
"""

    mslib.mscolab.seed
    ~~~~~~~~~~~~~~~~~~~~

    Seeder utility for database

    This file is part of MSS.

    :copyright: Copyright 2019 Shivashis Padhi
    :copyright: Copyright 2019-2022 by the MSS team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
import logging
import fs
from flask import Flask
import git
from sqlalchemy.exc import IntegrityError

from mslib.mscolab.conf import mscolab_settings
from mslib.mscolab.models import User, db, Permission, Operation


app = Flask(__name__, static_url_path='')


def add_all_users_to_all_operations(access_level='collaborator'):
    """ on db level we add all users as collaborator to all operations """
    app.config['SQLALCHEMY_DATABASE_URI'] = mscolab_settings.SQLALCHEMY_DB_URI
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
    db.init_app(app)
    with app.app_context():
        all_operations = Operation.query.all()
        all_path = [operation.path for operation in all_operations]
        db.session.close()
    for path in all_path:
        access_level = 'collaborator'
        if path == "TEMPLATE":
            access_level = 'admin'
        add_all_users_default_operation(path=path, access_level=access_level)


def add_all_users_default_operation(path='TEMPLATE', description="Operation to keep all users", access_level='admin'):
    """ on db level we add all users to the operation TEMPLATE for user handling"""
    app.config['SQLALCHEMY_DATABASE_URI'] = mscolab_settings.SQLALCHEMY_DB_URI
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False

    db.init_app(app)
    with app.app_context():
        operation_available = Operation.query.filter_by(path=path).first()
        if not operation_available:
            operation = Operation(path, description)
            db.session.add(operation)
            db.session.commit()
            with fs.open_fs(mscolab_settings.MSCOLAB_DATA_DIR) as file_dir:
                if not file_dir.exists(path):
                    file_dir.makedir(path)
                    file_dir.writetext(f'{path}/main.ftml', mscolab_settings.STUB_CODE)
                    # initiate git
                    r = git.Repo.init(fs.path.join(mscolab_settings.DATA_DIR, 'filedata', path))
                    r.git.clear_cache()
                    r.index.add(['main.ftml'])
                    r.index.commit("initial commit")

        operation = Operation.query.filter_by(path=path).first()
        op_id = operation.id
        user_list = User.query \
            .join(Permission, (User.id == Permission.u_id) & (Permission.op_id == op_id), isouter=True) \
            .add_columns(User.id, User.username) \
            .filter(Permission.u_id.is_(None))

        new_u_ids = [user.id for user in user_list]
        new_permissions = []
        for u_id in new_u_ids:
            if Permission.query.filter_by(u_id=u_id, op_id=op_id).first() is None:
                new_permissions.append(Permission(u_id, operation.id, access_level))
        db.session.add_all(new_permissions)
        try:
            db.session.commit()
            return True
        except IntegrityError as err:
            db.session.rollback()
            logging.debug(f"Error writing to db: {err}")
        db.session.close()


def delete_user(email):
    app.config['SQLALCHEMY_DATABASE_URI'] = mscolab_settings.SQLALCHEMY_DB_URI
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
    db.init_app(app)
    with app.app_context():
        user = User.query.filter_by(emailid=str(email)).first()
        if user:
            logging.info(f"User: {email} deleted from db")
            db.session.delete(user)
            db.session.commit()
            db.session.close()
            return True
        db.session.close()
        return False


def add_user(email, username, password):
    """
    on db level we add a user
    """
    app.config['SQLALCHEMY_DATABASE_URI'] = mscolab_settings.SQLALCHEMY_DB_URI
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
    db.init_app(app)

    template = f"""
    "MSCOLAB_mailid": "{email}",
    "MSCOLAB_password": "{password}",
"""
    with app.app_context():
        user_email_exists = User.query.filter_by(emailid=str(email)).first()
        user_name_exists = User.query.filter_by(username=str(username)).first()
        if not user_email_exists and not user_name_exists:
            db_user = User(email, username, password)
            db.session.add(db_user)
            db.session.commit()
            db.session.close()
            logging.info(f"Userdata: {email} {username} {password}")
            logging.info(template)
            return True
        else:
            logging.info(f"{user_name_exists} already in db")
    return False


def get_user(email):
    with app.app_context():
        return User.query.filter_by(emailid=str(email)).first()


def get_operation(operation_name):
    with app.app_context():
        return Operation.query.filter_by(path=operation_name).first()


def add_operation(operation_name, description):
    app.config['SQLALCHEMY_DATABASE_URI'] = mscolab_settings.SQLALCHEMY_DB_URI
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
    db.init_app(app)
    with app.app_context():
        operation_available = Operation.query.filter_by(path=operation_name).first()
        if not operation_available:
            operation = Operation(operation_name, description)
            db.session.add(operation)
            db.session.commit()
            with fs.open_fs(mscolab_settings.MSCOLAB_DATA_DIR) as file_dir:
                if not file_dir.exists(operation_name):
                    file_dir.makedir(operation_name)
                    file_dir.writetext(f'{operation_name}/main.ftml', mscolab_settings.STUB_CODE)
                    # initiate git
                    r = git.Repo.init(fs.path.join(mscolab_settings.DATA_DIR, 'filedata', operation_name))
                    r.git.clear_cache()
                    r.index.add(['main.ftml'])
                    r.index.commit("initial commit")
            return True
        else:
            return False


def delete_operation(operation_name):
    app.config['SQLALCHEMY_DATABASE_URI'] = mscolab_settings.SQLALCHEMY_DB_URI
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
    db.init_app(app)
    with app.app_context():
        operation = Operation.query.filter_by(path=operation_name).first()
        if operation:
            db.session.delete(operation)
            db.session.commit()
            db.session.close()
            return True
        db.session.close()
        return False


def add_user_to_operation(path=None, access_level='admin', emailid=None):
    """ on db level we add all users to the operation TEMPLATE for user handling"""
    if None in (path, emailid):
        return False
    app.config['SQLALCHEMY_DATABASE_URI'] = mscolab_settings.SQLALCHEMY_DB_URI
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False

    db.init_app(app)
    with app.app_context():
        operation = Operation.query.filter_by(path=path).first()
        if operation:

            user = User.query.filter_by(emailid=emailid).first()
            if user:
                new_permissions = [Permission(user.id, operation.id, access_level)]
                db.session.add_all(new_permissions)
                try:
                    db.session.commit()
                    return True
                except IntegrityError as err:
                    db.session.rollback()
                    logging.debug(f"Error writing to db: {err}")
                db.session.close()
    return False


def seed_data():
    app.config['SQLALCHEMY_DATABASE_URI'] = mscolab_settings.SQLALCHEMY_DB_URI
    app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
    db.init_app(app)

    with app.app_context():
        # create users
        users = [{
            'username': 'a',
            'id': 8,
            'password': 'a',
            'emailid': 'a'
        }, {
            'username': 'b',
            'id': 9,
            'password': 'b',
            'emailid': 'b'
        }, {
            'username': 'c',
            'id': 10,
            'password': 'c',
            'emailid': 'c'
        }, {
            'username': 'd',
            'id': 11,
            'password': 'd',
            'emailid': 'd'
        }, {
            'username': 'test1',
            'id': 12,
            'password': 'test1',
            'emailid': 'test1'
        }, {
            'username': 'test2',
            'id': 13,
            'password': 'test2',
            'emailid': 'test2'
        }, {
            'username': 'test3',
            'id': 14,
            'password': 'test3',
            'emailid': 'test3'
        }, {
            'username': 'test4',
            'id': 15,
            'password': 'test4',
            'emailid': 'test4'
        }, {
            'username': 'mscolab_user',
            'id': 16,
            'password': 'password',
            'emailid': 'mscolab_user'
        }, {
            'username': 'merge_waypoints_user',
            'id': 17,
            'password': 'password',
            'emailid': 'merge_waypoints_user'
        }]
        for user in users:
            db_user = User(user['emailid'], user['username'], user['password'])
            db_user.id = user['id']
            db.session.add(db_user)

        # create operations
        operations = [{
            'id': 1,
            'path': 'one',
            'description': 'a, b',
            'category': 'default'
        }, {
            'id': 2,
            'path': 'two',
            'description': 'b, c',
            'category': 'default'
        }, {
            'id': 3,
            'path': 'three',
            'description': 'a, c',
            'category': 'default'
        }, {
            'id': 4,
            'path': 'four',
            'description': 'd',
            'category': 'default'
        }, {
            'id': 5,
            'path': 'Admin_Test',
            'description': 'Operation for testing admin window',
            'category': 'default'
        }, {
            'id': 6,
            'path': 'test_mscolab',
            'description': 'Operation for testing mscolab main window',
            'category': 'default'
        }]
        for operation in operations:
            db_operation = Operation(operation['path'], operation['description'], operation['category'])
            db_operation.id = operation['id']
            db.session.add(db_operation)

        # create permissions
        permissions = [{
            'u_id': 8,
            'op_id': 1,
            'access_level': "creator"
        }, {
            'u_id': 9,
            'op_id': 1,
            'access_level': "collaborator"
        }, {
            'u_id': 9,
            'op_id': 2,
            'access_level': "creator"
        }, {
            'u_id': 10,
            'op_id': 2,
            'access_level': "collaborator"
        }, {
            'u_id': 10,
            'op_id': 3,
            'access_level': "creator"
        }, {
            'u_id': 8,
            'op_id': 3,
            'access_level': "collaborator"
        }, {
            'u_id': 10,
            'op_id': 1,
            'access_level': "viewer"
        }, {
            'u_id': 11,
            'op_id': 4,
            'access_level': 'creator'
        }, {
            'u_id': 8,
            'op_id': 4,
            'access_level': 'admin'
        }, {
            'u_id': 13,
            'op_id': 3,
            'access_level': 'viewer'
        }, {
            'u_id': 12,
            'op_id': 5,
            'access_level': 'creator'
        }, {
            'u_id': 12,
            'op_id': 3,
            'access_level': 'collaborator'
        }, {
            'u_id': 15,
            'op_id': 5,
            'access_level': 'viewer'
        }, {
            'u_id': 14,
            'op_id': 3,
            'access_level': 'collaborator'
        }, {
            'u_id': 15,
            'op_id': 3,
            'access_level': 'collaborator'
        }, {
            'u_id': 16,
            'op_id': 6,
            'access_level': 'creator'
        }, {
            'u_id': 17,
            'op_id': 6,
            'access_level': 'admin'
        }]
        for perm in permissions:
            db_perm = Permission(perm['u_id'], perm['op_id'], perm['access_level'])
            db.session.add(db_perm)
        db.session.commit()
        db.session.close()

    with fs.open_fs(mscolab_settings.MSCOLAB_DATA_DIR) as file_dir:
        file_paths = ['one', 'two', 'three', 'four', 'Admin_Test', 'test_mscolab']
        for file_path in file_paths:
            file_dir.makedir(file_path)
            file_dir.writetext(f'{file_path}/main.ftml', mscolab_settings.STUB_CODE)
            # initiate git
            r = git.Repo.init(fs.path.join(mscolab_settings.DATA_DIR, 'filedata', file_path))
            r.git.clear_cache()
            r.index.add(['main.ftml'])
            r.index.commit("initial commit")



# -*- coding: utf-8 -*-
"""

    mslib.mswms.mpl_lsec_styles
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Matplotlib linear section styles.

    This file is part of MSS.

    :copyright: Copyright 2021 May Baer
    :copyright: Copyright 2021-2022 by the MSS team, see AUTHORS.
    :license: APACHE-2.0, see LICENSE for details.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import numpy as np

from mslib.mswms.mpl_lsec import AbstractLinearSectionStyle
import mslib.utils.thermolib as thermolib
from mslib.utils.units import convert_to


class LS_DefaultStyle(AbstractLinearSectionStyle):
    """
    Style for single variables that require no further calculation
    """

    def __init__(self, driver, variable="air_temperature", filetype="ml"):
        super(AbstractLinearSectionStyle, self).__init__(driver=driver)
        self.variable = variable
        self.required_datafields = [(filetype, self.variable, None)]
        if filetype != "pl":
            self.required_datafields.insert(0, (filetype, "air_pressure", "Pa"))
        self.name = f"LS_Default_{variable}"
        self.title = f"{self.variable} Linear Plot"


class LS_RelativeHumdityStyle_01(AbstractLinearSectionStyle):
    """
    Linear plot of relative humidity.
    """

    name = "LS_RH01"
    title = "Relative Humdity (%) Linear Plot"
    abstract = "Relative humdity (%)"

    # Variables with the highest number of dimensions first (otherwise
    # MFDatasetCommonDims will throw an exception)!
    required_datafields = [
        ("ml", "air_pressure", "Pa"),
        ("ml", "air_temperature", "K"),
        ("ml", "specific_humidity", "kg/kg")]

    def _prepare_datafields(self):
        """
        Computes relative humdity.
        """
        self.data["relative_humidity"] = thermolib.rel_hum(
            self.data['air_pressure'], self.data["air_temperature"],
            self.data["specific_humidity"])
        self.variable = "relative_humidity"
        self.y_values = self.data[self.variable]
        self.unit = "%"


class LS_VerticalVelocityStyle_01(AbstractLinearSectionStyle):
    """
    Linear plot of vertical velocity.
    """

    name = "LS_W01"
    title = "Vertical Velocity (cm/s) Linear Plot"
    abstract = "Vertical velocity (cm/s)"

    # Variables with the highest number of dimensions first (otherwise
    # MFDatasetCommonDims will throw an exception)!
    required_datafields = [
        ("ml", "air_pressure", "Pa"),
        ("ml", "air_temperature", "K"),
        ("ml", "lagrangian_tendency_of_air_pressure", "Pa/s")]

    def _prepare_datafields(self):
        """
        Computes vertical velocity in cm/s.
        """
        self.data["upward_wind"] = convert_to(
            thermolib.omega_to_w(self.data["lagrangian_tendency_of_air_pressure"],
                                 self.data['air_pressure'], self.data["air_temperature"]),
            "m/s", "cm/s")
        self.variable = "upward_wind"
        self.y_values = self.data[self.variable]
        self.unit = "cm/s"


class LS_HorizontalVelocityStyle_01(AbstractLinearSectionStyle):
    """
    Linear plot of horizontal velocity.
    """

    name = "LS_HV01"
    title = "Horizontal Wind (m/s) Linear Plot"
    abstract = "Horizontal wind speed (m/s)"

    # Variables with the highest number of dimensions first (otherwise
    # MFDatasetCommonDims will throw an exception)!
    required_datafields = [
        ("ml", "air_pressure", "Pa"),
        ("ml", "eastward_wind", "m/s"),
        ("ml", "northward_wind", "m/s")]

    def _prepare_datafields(self):
        """
        Computes total horizontal wind speed.
        """
        self.data["horizontal_wind"] = np.hypot(
            self.data["eastward_wind"], self.data["northward_wind"])
        self.variable = "horizontal_wind"
        self.y_values = self.data[self.variable]
        self.unit = "m/s"



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui_mscolab_admin_window.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MscolabAdminWindow(object):
    def setupUi(self, MscolabAdminWindow):
        MscolabAdminWindow.setObjectName("MscolabAdminWindow")
        MscolabAdminWindow.resize(982, 756)
        self.centralwidget = QtWidgets.QWidget(MscolabAdminWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setContentsMargins(10, 5, 10, 10)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setContentsMargins(0, -1, -1, -1)
        self.horizontalLayout_6.setSpacing(10)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout_7.setContentsMargins(0, 8, -1, 8)
        self.horizontalLayout_7.setSpacing(6)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.usernameLabel = QtWidgets.QLabel(self.centralwidget)
        self.usernameLabel.setObjectName("usernameLabel")
        self.horizontalLayout_7.addWidget(self.usernameLabel)
        self.operationNameLabel = QtWidgets.QLabel(self.centralwidget)
        self.operationNameLabel.setObjectName("operationNameLabel")
        self.horizontalLayout_7.addWidget(self.operationNameLabel)
        self.verticalLayout.addLayout(self.horizontalLayout_7)
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(10)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.addUsersSearch = QtWidgets.QLineEdit(self.centralwidget)
        self.addUsersSearch.setObjectName("addUsersSearch")
        self.horizontalLayout_2.addWidget(self.addUsersSearch)
        self.selectAllAddBtn = QtWidgets.QPushButton(self.centralwidget)
        self.selectAllAddBtn.setObjectName("selectAllAddBtn")
        self.horizontalLayout_2.addWidget(self.selectAllAddBtn)
        self.deselectAllAddBtn = QtWidgets.QPushButton(self.centralwidget)
        self.deselectAllAddBtn.setObjectName("deselectAllAddBtn")
        self.horizontalLayout_2.addWidget(self.deselectAllAddBtn)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.addUsersTable = QtWidgets.QTableWidget(self.centralwidget)
        self.addUsersTable.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.addUsersTable.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.addUsersTable.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.addUsersTable.setObjectName("addUsersTable")
        self.addUsersTable.setColumnCount(1)
        self.addUsersTable.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.addUsersTable.setHorizontalHeaderItem(0, item)
        self.addUsersTable.horizontalHeader().setStretchLastSection(True)
        self.addUsersTable.verticalHeader().setVisible(False)
        self.verticalLayout.addWidget(self.addUsersTable)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setSpacing(10)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.addUsersPermission = QtWidgets.QComboBox(self.centralwidget)
        self.addUsersPermission.setObjectName("addUsersPermission")
        self.addUsersPermission.addItem("")
        self.addUsersPermission.addItem("")
        self.addUsersPermission.addItem("")
        self.horizontalLayout_3.addWidget(self.addUsersPermission)
        self.addUsersBtn = QtWidgets.QPushButton(self.centralwidget)
        self.addUsersBtn.setObjectName("addUsersBtn")
        self.horizontalLayout_3.addWidget(self.addUsersBtn)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_6.addLayout(self.verticalLayout)
        self.line = QtWidgets.QFrame(self.centralwidget)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout_6.addWidget(self.line)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setSpacing(6)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setSpacing(10)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_3 = QtWidgets.QLabel(self.centralwidget)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_8.addWidget(self.label_3)
        self.filterCategoryCb = QtWidgets.QComboBox(self.centralwidget)
        self.filterCategoryCb.setObjectName("filterCategoryCb")
        self.filterCategoryCb.addItem("")
        self.horizontalLayout_8.addWidget(self.filterCategoryCb)
        self.importPermissionsCB = QtWidgets.QComboBox(self.centralwidget)
        self.importPermissionsCB.setObjectName("importPermissionsCB")
        self.horizontalLayout_8.addWidget(self.importPermissionsCB)
        self.importPermissionsBtn = QtWidgets.QPushButton(self.centralwidget)
        self.importPermissionsBtn.setObjectName("importPermissionsBtn")
        self.horizontalLayout_8.addWidget(self.importPermissionsBtn)
        self.verticalLayout_2.addLayout(self.horizontalLayout_8)
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setSpacing(10)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.modifyUsersSearch = QtWidgets.QLineEdit(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.modifyUsersSearch.sizePolicy().hasHeightForWidth())
        self.modifyUsersSearch.setSizePolicy(sizePolicy)
        self.modifyUsersSearch.setObjectName("modifyUsersSearch")
        self.horizontalLayout_4.addWidget(self.modifyUsersSearch)
        self.modifyUsersPermissionFilter = QtWidgets.QComboBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.modifyUsersPermissionFilter.sizePolicy().hasHeightForWidth())
        self.modifyUsersPermissionFilter.setSizePolicy(sizePolicy)
        self.modifyUsersPermissionFilter.setObjectName("modifyUsersPermissionFilter")
        self.modifyUsersPermissionFilter.addItem("")
        self.modifyUsersPermissionFilter.addItem("")
        self.modifyUsersPermissionFilter.addItem("")
        self.modifyUsersPermissionFilter.addItem("")
        self.horizontalLayout_4.addWidget(self.modifyUsersPermissionFilter)
        self.selectAllModifyBtn = QtWidgets.QPushButton(self.centralwidget)
        self.selectAllModifyBtn.setObjectName("selectAllModifyBtn")
        self.horizontalLayout_4.addWidget(self.selectAllModifyBtn)
        self.deselectAllModifyBtn = QtWidgets.QPushButton(self.centralwidget)
        self.deselectAllModifyBtn.setObjectName("deselectAllModifyBtn")
        self.horizontalLayout_4.addWidget(self.deselectAllModifyBtn)
        self.horizontalLayout_4.setStretch(0, 3)
        self.horizontalLayout_4.setStretch(1, 1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.modifyUsersTable = QtWidgets.QTableWidget(self.centralwidget)
        self.modifyUsersTable.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.modifyUsersTable.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.modifyUsersTable.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.modifyUsersTable.setObjectName("modifyUsersTable")
        self.modifyUsersTable.setColumnCount(2)
        self.modifyUsersTable.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.modifyUsersTable.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.modifyUsersTable.setHorizontalHeaderItem(1, item)
        self.modifyUsersTable.horizontalHeader().setStretchLastSection(True)
        self.modifyUsersTable.verticalHeader().setVisible(False)
        self.verticalLayout_2.addWidget(self.modifyUsersTable)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setSpacing(10)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.modifyUsersPermission = QtWidgets.QComboBox(self.centralwidget)
        self.modifyUsersPermission.setObjectName("modifyUsersPermission")
        self.modifyUsersPermission.addItem("")
        self.modifyUsersPermission.addItem("")
        self.modifyUsersPermission.addItem("")
        self.horizontalLayout_5.addWidget(self.modifyUsersPermission)
        self.modifyUsersBtn = QtWidgets.QPushButton(self.centralwidget)
        self.modifyUsersBtn.setObjectName("modifyUsersBtn")
        self.horizontalLayout_5.addWidget(self.modifyUsersBtn)
        self.deleteUsersBtn = QtWidgets.QPushButton(self.centralwidget)
        self.deleteUsersBtn.setObjectName("deleteUsersBtn")
        self.horizontalLayout_5.addWidget(self.deleteUsersBtn)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6.addLayout(self.verticalLayout_2)
        self.gridLayout.addLayout(self.horizontalLayout_6, 0, 0, 1, 1)
        MscolabAdminWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MscolabAdminWindow)
        self.statusbar.setObjectName("statusbar")
        MscolabAdminWindow.setStatusBar(self.statusbar)
        self.actionCloseWindow = QtWidgets.QAction(MscolabAdminWindow)
        self.actionCloseWindow.setObjectName("actionCloseWindow")
        MscolabAdminWindow.addAction(self.actionCloseWindow)

        self.retranslateUi(MscolabAdminWindow)
        self.actionCloseWindow.triggered.connect(MscolabAdminWindow.close)
        QtCore.QMetaObject.connectSlotsByName(MscolabAdminWindow)

    def retranslateUi(self, MscolabAdminWindow):
        _translate = QtCore.QCoreApplication.translate
        MscolabAdminWindow.setWindowTitle(_translate("MscolabAdminWindow", "Manage Users"))
        self.usernameLabel.setText(_translate("MscolabAdminWindow", "Logged In: "))
        self.operationNameLabel.setText(_translate("MscolabAdminWindow", "Operation: "))
        self.label.setText(_translate("MscolabAdminWindow", "All Users Without Permission:"))
        self.addUsersSearch.setPlaceholderText(_translate("MscolabAdminWindow", "Search User"))
        self.selectAllAddBtn.setText(_translate("MscolabAdminWindow", "Select All"))
        self.deselectAllAddBtn.setText(_translate("MscolabAdminWindow", "Deselect All"))
        item = self.addUsersTable.horizontalHeaderItem(0)
        item.setText(_translate("MscolabAdminWindow", "Username"))
        self.addUsersPermission.setItemText(0, _translate("MscolabAdminWindow", "admin"))
        self.addUsersPermission.setItemText(1, _translate("MscolabAdminWindow", "collaborator"))
        self.addUsersPermission.setItemText(2, _translate("MscolabAdminWindow", "viewer"))
        self.addUsersBtn.setToolTip(_translate("MscolabAdminWindow", "Add the selected users to the operation"))
        self.addUsersBtn.setText(_translate("MscolabAdminWindow", "Add"))
        self.label_3.setText(_translate("MscolabAdminWindow", "Clone Operation Permissions:"))
        self.filterCategoryCb.setItemText(0, _translate("MscolabAdminWindow", "ANY"))
        self.importPermissionsBtn.setToolTip(_translate("MscolabAdminWindow", "Import permissions from another operation"))
        self.importPermissionsBtn.setText(_translate("MscolabAdminWindow", "Clone"))
        self.label_2.setText(_translate("MscolabAdminWindow", "All Users With Permission:"))
        self.modifyUsersSearch.setPlaceholderText(_translate("MscolabAdminWindow", "Search User"))
        self.modifyUsersPermissionFilter.setItemText(0, _translate("MscolabAdminWindow", "all"))
        self.modifyUsersPermissionFilter.setItemText(1, _translate("MscolabAdminWindow", "admin"))
        self.modifyUsersPermissionFilter.setItemText(2, _translate("MscolabAdminWindow", "collaborator"))
        self.modifyUsersPermissionFilter.setItemText(3, _translate("MscolabAdminWindow", "viewer"))
        self.selectAllModifyBtn.setText(_translate("MscolabAdminWindow", "Select All"))
        self.deselectAllModifyBtn.setText(_translate("MscolabAdminWindow", "Deselect All"))
        item = self.modifyUsersTable.horizontalHeaderItem(0)
        item.setText(_translate("MscolabAdminWindow", "Username"))
        item = self.modifyUsersTable.horizontalHeaderItem(1)
        item.setText(_translate("MscolabAdminWindow", "Permission"))
        self.modifyUsersPermission.setItemText(0, _translate("MscolabAdminWindow", "admin"))
        self.modifyUsersPermission.setItemText(1, _translate("MscolabAdminWindow", "collaborator"))
        self.modifyUsersPermission.setItemText(2, _translate("MscolabAdminWindow", "viewer"))
        self.modifyUsersBtn.setToolTip(_translate("MscolabAdminWindow", "Modify access permission of selected users"))
        self.modifyUsersBtn.setText(_translate("MscolabAdminWindow", "Modify"))
        self.deleteUsersBtn.setToolTip(_translate("MscolabAdminWindow", "Delete access permission of selected users"))
        self.deleteUsersBtn.setText(_translate("MscolabAdminWindow", "Delete"))
        self.actionCloseWindow.setText(_translate("MscolabAdminWindow", "actionCloseWindow"))
        self.actionCloseWindow.setShortcut(_translate("MscolabAdminWindow", "Ctrl+W"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mslib/msui/ui/ui_mscolab_connect_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MSColabConnectDialog(object):
    def setupUi(self, MSColabConnectDialog):
        MSColabConnectDialog.setObjectName("MSColabConnectDialog")
        MSColabConnectDialog.resize(478, 255)
        self.verticalLayout = QtWidgets.QVBoxLayout(MSColabConnectDialog)
        self.verticalLayout.setContentsMargins(12, 10, 10, 10)
        self.verticalLayout.setSpacing(5)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.urlLabel = QtWidgets.QLabel(MSColabConnectDialog)
        self.urlLabel.setObjectName("urlLabel")
        self.horizontalLayout_2.addWidget(self.urlLabel)
        self.urlCb = QtWidgets.QComboBox(MSColabConnectDialog)
        self.urlCb.setEditable(True)
        self.urlCb.setObjectName("urlCb")
        self.horizontalLayout_2.addWidget(self.urlCb)
        self.connectBtn = QtWidgets.QPushButton(MSColabConnectDialog)
        self.connectBtn.setAutoDefault(False)
        self.connectBtn.setObjectName("connectBtn")
        self.horizontalLayout_2.addWidget(self.connectBtn)
        self.horizontalLayout_2.setStretch(1, 1)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.line = QtWidgets.QFrame(MSColabConnectDialog)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.stackedWidget = QtWidgets.QStackedWidget(MSColabConnectDialog)
        self.stackedWidget.setObjectName("stackedWidget")
        self.loginPage = QtWidgets.QWidget()
        self.loginPage.setObjectName("loginPage")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.loginPage)
        self.gridLayout_3.setContentsMargins(100, 0, 100, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.addUserBtn = QtWidgets.QPushButton(self.loginPage)
        self.addUserBtn.setAutoDefault(False)
        self.addUserBtn.setObjectName("addUserBtn")
        self.gridLayout_3.addWidget(self.addUserBtn, 4, 1, 1, 1)
        self.loginBtn = QtWidgets.QPushButton(self.loginPage)
        self.loginBtn.setAutoDefault(False)
        self.loginBtn.setObjectName("loginBtn")
        self.gridLayout_3.addWidget(self.loginBtn, 3, 0, 1, 2)
        self.loginPasswordLe = QtWidgets.QLineEdit(self.loginPage)
        self.loginPasswordLe.setEchoMode(QtWidgets.QLineEdit.Password)
        self.loginPasswordLe.setObjectName("loginPasswordLe")
        self.gridLayout_3.addWidget(self.loginPasswordLe, 2, 0, 1, 2)
        self.loginEmailLe = QtWidgets.QLineEdit(self.loginPage)
        self.loginEmailLe.setObjectName("loginEmailLe")
        self.gridLayout_3.addWidget(self.loginEmailLe, 1, 0, 1, 2)
        self.clickNewUserLabel = QtWidgets.QLabel(self.loginPage)
        self.clickNewUserLabel.setObjectName("clickNewUserLabel")
        self.gridLayout_3.addWidget(self.clickNewUserLabel, 4, 0, 1, 1)
        self.loginTopicLabel = QtWidgets.QLabel(self.loginPage)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.loginTopicLabel.setFont(font)
        self.loginTopicLabel.setObjectName("loginTopicLabel")
        self.gridLayout_3.addWidget(self.loginTopicLabel, 0, 0, 1, 2, QtCore.Qt.AlignHCenter)
        self.stackedWidget.addWidget(self.loginPage)
        self.newuserPage = QtWidgets.QWidget()
        self.newuserPage.setObjectName("newuserPage")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.newuserPage)
        self.gridLayout_2.setContentsMargins(50, 0, 50, 0)
        self.gridLayout_2.setSpacing(5)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.newUsernameLe = QtWidgets.QLineEdit(self.newuserPage)
        self.newUsernameLe.setObjectName("newUsernameLe")
        self.gridLayout_2.addWidget(self.newUsernameLe, 1, 1, 1, 1)
        self.newPasswordLabel = QtWidgets.QLabel(self.newuserPage)
        self.newPasswordLabel.setObjectName("newPasswordLabel")
        self.gridLayout_2.addWidget(self.newPasswordLabel, 3, 0, 1, 1, QtCore.Qt.AlignRight)
        self.newConfirmPasswordLabel = QtWidgets.QLabel(self.newuserPage)
        self.newConfirmPasswordLabel.setObjectName("newConfirmPasswordLabel")
        self.gridLayout_2.addWidget(self.newConfirmPasswordLabel, 4, 0, 1, 1, QtCore.Qt.AlignRight)
        self.newUserTopicLabel = QtWidgets.QLabel(self.newuserPage)
        font = QtGui.QFont()
        font.setPointSize(16)
        self.newUserTopicLabel.setFont(font)
        self.newUserTopicLabel.setObjectName("newUserTopicLabel")
        self.gridLayout_2.addWidget(self.newUserTopicLabel, 0, 1, 1, 1, QtCore.Qt.AlignLeft)
        self.newEmailLe = QtWidgets.QLineEdit(self.newuserPage)
        self.newEmailLe.setObjectName("newEmailLe")
        self.gridLayout_2.addWidget(self.newEmailLe, 2, 1, 1, 1)
        self.newEmailLabel = QtWidgets.QLabel(self.newuserPage)
        self.newEmailLabel.setObjectName("newEmailLabel")
        self.gridLayout_2.addWidget(self.newEmailLabel, 2, 0, 1, 1, QtCore.Qt.AlignRight)
        self.newUserBb = QtWidgets.QDialogButtonBox(self.newuserPage)
        self.newUserBb.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.newUserBb.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.newUserBb.setObjectName("newUserBb")
        self.gridLayout_2.addWidget(self.newUserBb, 5, 1, 1, 1, QtCore.Qt.AlignLeft)
        self.newPasswordLe = QtWidgets.QLineEdit(self.newuserPage)
        self.newPasswordLe.setEchoMode(QtWidgets.QLineEdit.Password)
        self.newPasswordLe.setObjectName("newPasswordLe")
        self.gridLayout_2.addWidget(self.newPasswordLe, 3, 1, 1, 1)
        self.newUsernameLabel = QtWidgets.QLabel(self.newuserPage)
        self.newUsernameLabel.setObjectName("newUsernameLabel")
        self.gridLayout_2.addWidget(self.newUsernameLabel, 1, 0, 1, 1, QtCore.Qt.AlignRight)
        self.newConfirmPasswordLe = QtWidgets.QLineEdit(self.newuserPage)
        self.newConfirmPasswordLe.setEchoMode(QtWidgets.QLineEdit.Password)
        self.newConfirmPasswordLe.setObjectName("newConfirmPasswordLe")
        self.gridLayout_2.addWidget(self.newConfirmPasswordLe, 4, 1, 1, 1)
        self.stackedWidget.addWidget(self.newuserPage)
        self.httpAuthPage = QtWidgets.QWidget()
        self.httpAuthPage.setObjectName("httpAuthPage")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.httpAuthPage)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.httpTopicLabel = QtWidgets.QLabel(self.httpAuthPage)
        self.httpTopicLabel.setObjectName("httpTopicLabel")
        self.verticalLayout_4.addWidget(self.httpTopicLabel)
        self.httpInfoLabel = QtWidgets.QLabel(self.httpAuthPage)
        self.httpInfoLabel.setObjectName("httpInfoLabel")
        self.verticalLayout_4.addWidget(self.httpInfoLabel)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.httpUsernameLabel = QtWidgets.QLabel(self.httpAuthPage)
        self.httpUsernameLabel.setObjectName("httpUsernameLabel")
        self.gridLayout.addWidget(self.httpUsernameLabel, 0, 0, 1, 1)
        self.httpPasswordLe = QtWidgets.QLineEdit(self.httpAuthPage)
        self.httpPasswordLe.setEchoMode(QtWidgets.QLineEdit.Password)
        self.httpPasswordLe.setObjectName("httpPasswordLe")
        self.gridLayout.addWidget(self.httpPasswordLe, 1, 1, 1, 1)
        self.httpPasswordLabel = QtWidgets.QLabel(self.httpAuthPage)
        self.httpPasswordLabel.setObjectName("httpPasswordLabel")
        self.gridLayout.addWidget(self.httpPasswordLabel, 1, 0, 1, 1)
        self.httpUsernameLe = QtWidgets.QLineEdit(self.httpAuthPage)
        self.httpUsernameLe.setObjectName("httpUsernameLe")
        self.gridLayout.addWidget(self.httpUsernameLe, 0, 1, 1, 1)
        self.verticalLayout_4.addLayout(self.gridLayout)
        self.httpBb = QtWidgets.QDialogButtonBox(self.httpAuthPage)
        self.httpBb.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.httpBb.setObjectName("httpBb")
        self.verticalLayout_4.addWidget(self.httpBb)
        self.stackedWidget.addWidget(self.httpAuthPage)
        self.verticalLayout.addWidget(self.stackedWidget)
        self.line_2 = QtWidgets.QFrame(MSColabConnectDialog)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout.addWidget(self.line_2)
        self.statusHL = QtWidgets.QHBoxLayout()
        self.statusHL.setContentsMargins(-1, 0, -1, -1)
        self.statusHL.setObjectName("statusHL")
        self.statusLabel = QtWidgets.QLabel(MSColabConnectDialog)
        self.statusLabel.setStyleSheet("")
        self.statusLabel.setObjectName("statusLabel")
        self.statusHL.addWidget(self.statusLabel)
        self.statusHL.setStretch(0, 1)
        self.verticalLayout.addLayout(self.statusHL)

        self.retranslateUi(MSColabConnectDialog)
        self.stackedWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MSColabConnectDialog)
        MSColabConnectDialog.setTabOrder(self.urlCb, self.connectBtn)
        MSColabConnectDialog.setTabOrder(self.connectBtn, self.loginEmailLe)
        MSColabConnectDialog.setTabOrder(self.loginEmailLe, self.loginPasswordLe)
        MSColabConnectDialog.setTabOrder(self.loginPasswordLe, self.loginBtn)
        MSColabConnectDialog.setTabOrder(self.loginBtn, self.addUserBtn)
        MSColabConnectDialog.setTabOrder(self.addUserBtn, self.newUsernameLe)
        MSColabConnectDialog.setTabOrder(self.newUsernameLe, self.newEmailLe)
        MSColabConnectDialog.setTabOrder(self.newEmailLe, self.newPasswordLe)
        MSColabConnectDialog.setTabOrder(self.newPasswordLe, self.newConfirmPasswordLe)
        MSColabConnectDialog.setTabOrder(self.newConfirmPasswordLe, self.httpUsernameLe)
        MSColabConnectDialog.setTabOrder(self.httpUsernameLe, self.httpPasswordLe)

    def retranslateUi(self, MSColabConnectDialog):
        _translate = QtCore.QCoreApplication.translate
        MSColabConnectDialog.setWindowTitle(_translate("MSColabConnectDialog", "Connect to MSColab"))
        self.urlLabel.setText(_translate("MSColabConnectDialog", "MSColab URL:"))
        self.urlCb.setToolTip(_translate("MSColabConnectDialog", "Enter Mscolab Server URL"))
        self.connectBtn.setToolTip(_translate("MSColabConnectDialog", "Connect to entered URL"))
        self.connectBtn.setText(_translate("MSColabConnectDialog", "Connect"))
        self.addUserBtn.setToolTip(_translate("MSColabConnectDialog", "Add new user to the server"))
        self.addUserBtn.setText(_translate("MSColabConnectDialog", "Add user"))
        self.loginBtn.setToolTip(_translate("MSColabConnectDialog", "Login using entered credentials"))
        self.loginBtn.setText(_translate("MSColabConnectDialog", "Login"))
        self.loginPasswordLe.setPlaceholderText(_translate("MSColabConnectDialog", "Password"))
        self.loginEmailLe.setPlaceholderText(_translate("MSColabConnectDialog", "Email ID"))
        self.clickNewUserLabel.setText(_translate("MSColabConnectDialog", "Click here if new user"))
        self.loginTopicLabel.setText(_translate("MSColabConnectDialog", "Login Details:"))
        self.newUsernameLe.setPlaceholderText(_translate("MSColabConnectDialog", "John Doe"))
        self.newPasswordLabel.setText(_translate("MSColabConnectDialog", "Password:"))
        self.newConfirmPasswordLabel.setText(_translate("MSColabConnectDialog", "Confirm Password:"))
        self.newUserTopicLabel.setText(_translate("MSColabConnectDialog", "New User Details"))
        self.newEmailLe.setPlaceholderText(_translate("MSColabConnectDialog", "johndoe@gmail.com"))
        self.newEmailLabel.setText(_translate("MSColabConnectDialog", "Email:"))
        self.newPasswordLe.setPlaceholderText(_translate("MSColabConnectDialog", "New Password"))
        self.newUsernameLabel.setText(_translate("MSColabConnectDialog", "Username:"))
        self.newConfirmPasswordLe.setPlaceholderText(_translate("MSColabConnectDialog", "Confirm New Password"))
        self.httpTopicLabel.setText(_translate("MSColabConnectDialog", "HTTP Server Authentication"))
        self.httpInfoLabel.setText(_translate("MSColabConnectDialog", "The server you are trying to connect requires a username and a password:"))
        self.httpUsernameLabel.setText(_translate("MSColabConnectDialog", "Username:"))
        self.httpPasswordLe.setPlaceholderText(_translate("MSColabConnectDialog", "Server Auth Password"))
        self.httpPasswordLabel.setText(_translate("MSColabConnectDialog", "Password:"))
        self.httpUsernameLe.setPlaceholderText(_translate("MSColabConnectDialog", "Server Auth Username"))
        self.statusLabel.setText(_translate("MSColabConnectDialog", "Status:"))
